#define MAIN
#include "main.h"

// Clock settings
// SYSCLK = 40 Mhz, PBCLK = 40 Mhz, REFCLK = 16 Mhz.

// Primary oscillator setting, XT for 8 MHz (between 3 and 10)
#pragma config POSCMOD = XT
// Primary Osc with PLL
#pragma config FNOSC = PRIPLL
#pragma config FPLLIDIV = DIV_2
#pragma config FPLLMUL = MUL_20
#pragma config FPLLODIV = DIV_2
#pragma config FPBDIV = DIV_1

// MISC disabled
#pragma config FSOSCEN = OFF
#pragma config IESO = OFF
#pragma config OSCIOFNC = OFF
#pragma config FCKSM = CSDCMD
#pragma config JTAGEN = OFF
#pragma config ICESEL = ICS_PGx1
#pragma config DEBUG = OFF
#pragma config PMDL1WAY = OFF
#pragma config IOL1WAY = OFF
#pragma config FUSBIDIO = OFF
#pragma config FVBUSONIO = OFF

// WDT disabled
#pragma config WDTPS = PS1, FWDTEN = OFF, WINDIS = OFF, FWDTWINSZ = WISZ_25

// protection all off
#pragma config PWP = OFF, BWP = OFF, CP = OFF

void main(){
	// Initialization routines
	loadTape();
	ps2_init();
	initNtsc();
	init8255();
	init8253();
	resetZ80();
	INTEnableSystemSingleVectoredInt();
	INTEnableSystemMultiVectoredInt();
	// Timer1 settings, ON=1, SIDL=1, TWDIS=0, TGATE=0, 1:1 prescale, TCS=0
	PR1=0xFFFF;
	TMR1=0;
	g_timer1=0;
	IFS0bits.T1IF=0;
	T1CON=0xA000;
	// Timer1 interrupt: priority 4
	IPC1bits.T1IP=4;
	IPC1bits.T1IS=0;
	IEC0bits.T1IE=1;
 
	// Execute the Z80 code, eternally.
	// This produces ~2 MHz speed.
	execZ80infinite();
	/*while(1){
		// Wait until next timing to execute Z80 code
		// while(TMR1<g_timer1){ }
		// Now, execute the Z80 code.
		// Note that g_timer1 will increment due to each code's T cycles
		execZ80code();
	}*/
}

#pragma interrupt timer1Int IPL4SOFT vector 4

void timer1Int(){
static i=0;
static j=0;
	// This occurs every 1.6384 msec (610 Hz).
	IFS0CLR=_IFS0_T1IF_MASK; //IFS0bits.T1IF=0;

/*if (++i<1500) return;
i=0;
j++;
VRAM[0]=0x20+(j/100);
VRAM[1]=0x20+(j-(j/100)*100)/10;
VRAM[2]=0x20+(j-(j/10)*10);
REFOCONbits.ON=0;
REFOTRIMbits.ROTRIM=j;
REFOCONbits.ON=1;
return;*/

	//g_timer1&=0xFFFF; // Currently, Z80 speed is not monitored.

	// Environmental supports follow.

	// Two 555 timers for tempo and prompt
	if (!((++tempo_8255)&0x7)) {
		read825x[8]^=1; // tempo
	}
	if (!((++prompt_8255)&0xff)) {
		read825x[2]^=0x40; // prompt
	}

	// Emulate 8253 timer 2
	if (IFS0bits.T5IF) inc8253timer2();

	// PS2 key input
	if (0x200000<ps2status) {
		if ((ps2data&0x200800)==0x200000) {
			// Parity check is not done.
			ps2_received((ps2data>>12)&0xff);			
		}
		ps2status=0x800;
		ps2data&=0x7ff;
	}
	if (ps2status==0x800) {
		if ((ps2data&0x401)==0x400) {
			// Parity check is not done.
			ps2_received((ps2data>>1)&0xff);
		}
		ps2status=1;
		ps2data=0;
	}
}