#include "main.h"
#include "cgrom.h"

/*
	Raster frequency is 15734 hz, so one raster takes 63.56 usecs.
	This corresponds to 2542.265 clocks of 40 Mhz CPU frequency.
*/

/*	Minimum time between two interruption events is 2.3 usecs,
	that corresponds to 92 colcks.
*/

#define sync LATBbits.LATB13
#define toggleSync() LATBINV=0x2000


void initNtsc(){
	unsigned long i;
	for (i=0;i<0x0400;i++) {
		VRAM[i]=i&0xFF;
	}
	g_ntscVisible=1;
	g_Vsync=1;
	sync=1;

	// RB13 and RA4 are output pins
	TRISAbits.TRISA4=0;
	TRISBbits.TRISB13=0;

	// Timer2 is used for sync signal
	// On, Stop in Idle Mode, PBCLK, 1:1 prescale, 16-bit mode
	// Enable interrupt as priority 7
	TMR2=0;
	PR2=0xFFFF;
	T2CON=0xA000;
	T2CONbits.ON=1;
	IPC2bits.T2IP=7;
	IPC2bits.T2IS=0;
	IFS0bits.T2IF=0;
	IEC0bits.T2IE=1;

	// Timer4 is used for video signal
	// Off, Stop in Idle Mode, PBCLK, 1:2 prescale, 16-bit mode
	// Enable interrupt as priority 6
	TMR4=0;
	PR4=0xFFFF;
	T4CON=0x2010;
	IPC4bits.T4IP=6;
	IPC4bits.T4IS=0;
	IFS0bits.T4IF=0;
	IEC0bits.T4IE=1;

	// REFCLK setting for 16 MHz.
	// RODIV=1 and ROTRIM=128 for N=2, M=256 in (N+M/512).???
	REFOCONbits.ACTIVE=0;
	REFOCONbits.ROSEL=0;//SYSCLK
	REFOCONbits.RODIV=1;
	// 132 is better. 160 is currently best. 
	// 96 or 114 is stable, but...
	REFOTRIMbits.ROTRIM=114;
	REFOCONbits.ON=1;
	REFOCONbits.ACTIVE=1;
//*/

	// SPI2 module settings follow
	// REFCLOCK, Enable SPI2, Stop in Idle Mode, Enable SDO, 32 bit mode,
	// SS pin not used, Master mode, Disable SDI
	// SPI clock=REFCLK/(2x(SPI2BRG+1))=8 Mhz. (REFCLK=16 Mhz; SPI2BRG=0)
	// All extended setting are off (SPI2CON2).

	SPI2CON=0x0180A83C;
	SPI2CON2=0x0300;
	SPI2BRG=0;
	// Output SDO2 to RA4
	RPA4R=0x04;

}

/*	ntscSyncInt() is an interrupt handler
	Interrupt occur via Timer2 (vector 8)
*/
#pragma interrupt ntscSyncInt IPL7SOFT vector 8

/*	ntscVideoInt() is an interrupt handler
	Interrupt occur via Timer4 (vector 16)
*/
#pragma interrupt ntscVideoInt IPL6SOFT vector 16

/*	Sync signals:
	raster 1-3: L for 2.3 usecs, H for 29.5 usec, L for 2.3 usecs, H for 29.5 usec
	(92)          (92)
	2.3u__________2.3u__________                 92+1179+92+1179=2542
	|__|          |__|          
	       29.5u         29.5u
          (1179)        (1179)

	raster 4-6: L for 27.1 usecs, H for 4.7 usec, L for 27.1 usecs, H for 4.7 usec
      (1083)        (1083)
	   27.1u    __   27.1u    __                 1083+188+1083+188=2542
	|__________|  |__________|  
	           4.7u          4.7u
              (188)         (188)

	raster 7-9: L for 2.3 usecs, H for 29.5 usec, L for 2.3 usecs, H for 29.5 usec
	(92)          (92)
	2.3u__________2.3u__________                 92+1179+92+1179=2542
	|__|          |__|          
	       29.5u         29.5u
          (1179)        (1179)

	raster 10-262: H for 1.5 usecs, L for 4.7 usecs, H for 57.4 usecs
      (188)
	___4.7u_____________________                 60+188+2294=2542
	   |__|          
	1.5u          57.4u
    (60)         (2294)

	This must produces 16275 Hz sync signal.
	Following intterupt function takes about 1 micro second.
	This will be 0.5 micro second if IPL7SRS will be used.
*/

const unsigned short ntscSyncTiming[]={
// raster 1-3
	92-1,1179-1,92-1,1179-1,92-1,1179-1,92-1,1179-1,92-1,1179-1,92-1,1179-1,
// raster 4-6
	1083-1,188-1,1083-1,188-1,1083-1,188-1,1083-1,188-1,1083-1,188-1,1083-1,188-1,
// raster 7-9
	92-1,1179-1,92-1,1179-1,92-1,1179+60-1,92-1,1179-1,92-1,1179-1,92-1,1179+60-1,
// raster 10-259
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,188-1,2294+60-1,
// raster 260-262
	188-1,2294+60-1,188-1,2294+60-1,188-1,2294-1

};
void ntscSyncInt(){
	static unsigned long status=0;
	unsigned int i16;
	IFS0CLR=_IFS0_T2IF_MASK; //IFS0bits.T2IF=0;
	toggleSync();
	PR2=i16=ntscSyncTiming[status++];
	if (i16==(2294-1)) {
		// The end of the sequence of statuses.
		// Reset parameter.
		sync=1;
		status=0;
		// Trigger Timer4 for video signal
		// Note that video signal starts from raster 42.
		// Also note that timer for video works at 20 Mhz.
		TMR4=TMR2>>1;
		PR4=(2294+2542*41+60+188)/2+130-1; // Needs to be tuned to define horizontal positoin.
		T4CONSET=_T4CON_ON_MASK; //T4CONbits.ON=1;
		g_Vsync=0; // Pseudo vertical sync signal starts here.
	}
}
/*	Video signal construction
	1 raster: 1271 cycles of timer working at 20 Mhz (20 clocks/usec; 20 clocks/8 dotes)
	status:     1 2 3 4 5 6 7 8 9 101112 
	            | | | | | | | | | | | |
              (94)
	       ___4.7u_____________________
	          |__|          
	       1.5u          57.4u
           (30)         (1147)
	// This routine takes about 2 micro seconds.
	// This will take about 1 micro second if IPL7SRS will be used.
*/
void ntscVideoInt(){
	static unsigned long status=1;
	static unsigned long addr=0;
	static unsigned long fontLine=0;
	unsigned long i1,i2,i3,i4;
	static char* cgrom_line;
	IFS0CLR=_IFS0_T4IF_MASK; //IFS0bits.T4IF=0;
	switch(status){
		case 1:
			status=2;
			g_Vsync=1; // Pseudo vertical sync signal ends here.
			if (!g_ntscVisible) {
				T4CONCLR=_T4CON_ON_MASK; //T4CONbits.ON=0;
			} else {
				PR4=78; // Needs to be tuned (must be less than 80, more than 71.1).
				cgrom_line=&cgrom[fontLine];
			}
			SPI2BUF=0;
			break;
		case 12:
			status=1;
			if (fontLine<7*256) {
				fontLine+=256;
				addr-=40;
			} else {
				fontLine=0;
				if (999<addr) {
					// Video signal was already end. Let's stop timer
					T4CONCLR=_T4CON_ON_MASK; //T4CONbits.ON=0;
					// Reset parameters
					addr=0;
					fontLine=0;
					break;
				}
			}
			PR4=1271-(78+1)*11-1;
			break;
		default:
			status++;
			i1=(0xFF&cgrom_line[VRAM[addr+0]])<<24;
			i2=(0xFF&cgrom_line[VRAM[addr+1]])<<16;
			i3=(0xFF&cgrom_line[VRAM[addr+2]])<<8;
			i4=(0xFF&cgrom_line[VRAM[addr+3]]);
			SPI2BUF=i1|i2|i3|i4;
			addr+=4;
			break;
	}
}