/*	Environment(machine)-specific basic functions follow 
	These are specified to access to MZ-80K (with 28 KB RAM) memory map.
	I/O ports are connected to PORTB of PIC32MX150F128B.
	To emulate the other computer, these must be modified.
*/

// _getCode() macro (as inline function) that fetches a code using PC, and increment PC.
#define _getCode() ((z80PC<0x1000)\
	?(\
		monitor[(z80PC++)]\
	):(\
		(z80PC<0x8000)\
		?(\
			RAM[(z80PC++)-0x1000]\
		):(\
			(z80PC<0xD000)\
			?(\
				0xC7\
			):(\
				(z80PC<0xE000)\
				?(\
					VRAM[(z80PC++)&0x03FF]\
				):(\
					0xC7\
				)\
			)\
		)\
	))
UINT8 getCode(){
	// Using getCode() as inline function for all will use much memory to exceed the maximum.
	return _getCode();
}

// readMemory() function uses 34 cycles max, 21 cycles typical
unsigned char readMemory(UINT16 addr){
	addr&=0xffff;
	if (addr<0x1000) return monitor[addr];
	if (addr<0x8000) return RAM[addr-0x1000];
	if (addr<0xD000) return 0xC7;
	if (addr<0xE000) return VRAM[addr & 0x03FF];
	if (addr<0xE004) {
		// 8255
		read825x[2]&=0x7f;
		read825x[2]|=g_Vsync?0x80:0; // Wait for V blank
		return read825x[addr & 0x000F];
	}
	if (addr<0xE008) {
		// 8253
		return read8253(addr);
	}
	if (addr<0xE009) {
		// Tempo
		return read825x[8];
	}
	return 0xC7; // RST00
}

// writeMemory() function takes 44 cycles max, 31 cycles typical.
void writeMemory(UINT16 addr, UINT8 data){
	addr&=0xffff;
	data&=0xff;
	if (addr<0x1000) return; // ROM region cannot be written.
	if (addr<0x8000) {
		RAM[addr-0x1000] = data;
		return;
	}
	if (addr<0xD000) return;
	if (addr<0xE000) {
		VRAM[addr & 0x03FF] = data;
		return;
	}
	if (addr<0xE004) {
		write8255(addr,data);
		return;
	}
	if (addr<0xE008) {
		write8253(addr,data);
		return;
	}
	if (addr<0xE009) {
		// on-off sound
		T3CONbits.ON=(data&0x01);
		return;
	}
}
// readIO() function
unsigned char readIO(UINT8 addrL, UINT8 addrH){
	return 0xff;

	switch(addrL){
		case 0x80: // PORTB0-7
			return PORTB;
		case 0x81: // PORTB8-15
			return PORTB>>8;
		case 0x82: // LATB0-7
			return LATB;
		case 0x83: // LATB8-15
			return LATB>>8;
		case 0x84: // TRISB0-7
			return TRISB;
		case 0x85: // TRISB8-15
			return TRISB>>8;
		default:
			return 0x00;
	}
}
// writeIO() function
void writeIO(UINT8 addrL, UINT8 addrH, UINT8 data){
	return;

	switch(addrL){
		case 0x80: // PORTB0-7
			PORTB=PORTB&0xFF00 | data;
			break;
		case 0x81: // PORTB8-15
			PORTB=PORTB&0x00FF | (((UINT16)data)<<8);
			break;
		case 0x82: // LATB0-7
			LATB=LATB&0xFF00 | data;
			break;
		case 0x83: // LATB8-15
			LATB=LATB&0x00FF | (((UINT16)data)<<8);
			break;
		case 0x84: // TRISB0-7
			TRISB=TRISB&0xFF00 | data;
			break;
		case 0x85: // TRISB8-15
			TRISB=TRISB&0x00FF | (((UINT16)data)<<8);
			break;
		default:
			break;
	}
}

/*	End of environment-specific basic functions */
