/*	Z-80 emulator for microcontrollers.
	Written by Katsumi.
	This code is provided under the license of GPL v2.
	To emulate Z80 function, call execZ80codes() periodically.
	You will also know the next timing of call through setTx() callback 
	functions (see z80.h).
*/

#include "main.h"
#include "monitor.h"
#include "z80.h"
#include "z80macros.h"
#include "z80functions.h"
#include "peripheral.h"

/* Local prototypings */
void execZ80(UINT8 code);
void codeFF();

/* Reset */
void resetZ80(){
	// SP
	loadSP(0x10f0);
	// PC
	//loadPC(0x1200);
	loadPC(0x0000);
	// I and R
	loadI(0x00);
	loadR(0x00);
	// Disable interrupt and set interrupt mode to 0
	loadIM(0);
	loadIFF1(0);
	loadIFF2(0);
	z80irq=0;
	z80nrq=0;
}

/* Interrupts */
void intZ80(UINT8 i8){
	if (!flagIFF1) return;
	loadIFF1(0);
	loadIFF2(0);
	z80irq=1;
	z80intCode=i8;
}
void nmrZ80(){
	loadIFF1(0);
	z80irq=1;
	z80nrq=1;
}
void doZ80Int(){
	UINT16 addr;
	z80irq=0;
	if (z80nrq) {
		// NMR
		z80nrq=0;
		loadSP(regSP-2);
		writeMemory(regSP,regPC&0xFF);
		writeMemory(regSP+1,regPC>>8);
		loadPC(0x0066);
	} else {
		// Mascable interrupt
		switch (regIM) {
			case 0:
				execZ80(z80intCode);
				return;
			case 2:
				addr=(regI<<8)|z80intCode;
				loadSP(regSP-2);
				writeMemory(regSP,regPC&0xFF);
				writeMemory(regSP+1,regPC>>8);
				loadPC(readMemory(addr)|(readMemory(addr+1)<<8));
				return;
			case 1:
			default:
				codeFF();
				return;
		}
	}	
}

/*	Code emulation routines follow */
void codeVOID(){
	//NOP with longest T cycles
	setT23();
}
void code00(){
	//NOP
	setT4();
}
void code01(){
	//LD BC,nn
	setT10();
	loadC(getCode());
	loadB(getCode());
}
void code02(){
	//LD (BC),A
	setT7();
	writeMemory(regBC,regA);
}
void code03(){
	//INC BC
	//Note that 16-bit increment routine does not change flags.
	setT6_5();
	loadBC(regBC+1);
}
void code04(){
	UINT8 i8;
	//INC B
	setT4_5();
	i8=regB+1;
	loadB(i8);
	flag8inc(i8);
}
void code05(){
	UINT8 i8;
	//DEC B
	setT4_5();
	i8=regB-1;
	loadB(i8);
	flag8dec(i8);
}
void code06(){
	//LD B,n
	setT7();
	loadB(getCode());
}
void code07(){
	//RLCA  C <- A7<-A0 <- A7
	setT4();
	if (regA&0x80) {
		setCflag();
		loadA((regA<<1) | 0x01);
	} else {
		clearCflag();
		loadA(regA<<1);
	}
	clearHflag();
	clearNflag();	
}
#ifdef Z80
void code08(){
	UINT16 i16;
	//EX AF,AFf
	setT4();
	i16=regAF;
	loadAF(regAFprime);
	loadAFprime(i16);
}
#endif
void code09(){
	//ADD HL,BC
	setT11_10();
	z80ADD16(regBC);
}
void code0A(){
	//LD A,(BC)
	setT7();
	loadA(readMemory(regBC));
}
void code0B(){
	//DEC BC
	setT6_5();
	loadBC(regBC-1);
}
void code0C(){
	UINT8 i8;
	//INC C
	setT4_5();
	i8=regC+1;
	loadC(i8);
	flag8inc(i8);
}
void code0D(){
	UINT8 i8;
	//DEC C
	setT4_5();
	i8=regC-1;
	loadC(i8);
	flag8dec(i8);
}
void code0E(){
	//LD C,n
	setT7();
	loadC(getCode());
}
void code0F(){
	//RRCA A0 -> A7->A0 ->C
	setT4();
	if (regA&0x01) {
		setCflag();
		loadA((regA>>1) | 0x80);
	} else {
		clearCflag();
		loadA(regA>>1);
	}
	clearHflag();
	clearNflag();	
}
#ifdef Z80
void code10(){
	UINT8 i8;
	//DJNZ (PC+e)
	i8=regB-1;
	if (i8) {
		setT13();
		loadB(i8);
		z80JR(i8);
	} else {
		setT8();
		loadB(i8);
		getCode();
	}
}
#endif
void code11(){
	//LD DE,nn
	setT10();
	loadE(getCode());
	loadD(getCode());
}
void code12(){
	//LD (DE),A
	setT7();
	writeMemory(regDE,regA);
}
void code13(){
	//INC DE
	setT6_5();
	loadDE(regDE+1);
}
void code14(){
	UINT8 i8;
	//INC D
	setT4_5();
	i8=regD+1;
	loadD(i8);
	flag8inc(i8);
}
void code15(){
	UINT8 i8;
	//DEC D
	setT4_5();
	i8=regD-1;
	loadD(i8);
	flag8dec(i8);
}
void code16(){
	//LD D,n
	setT7();
	loadD(getCode());
}
void code17(){
	UINT8 i8;
	//RLA C<- A7<-A0 <-C
	setT4();
	i8=flagC;
	if (regA&0x80) setCflag();
	else clearCflag();		
	if (i8) loadA((regA<<1) | 0x01);
	else loadA(regA<<1);
	clearHflag();
	clearNflag();	
}
#ifdef Z80
void code18(){
	UINT8 i8;
	//JR (PC+e)
	setT12();
	z80JR(i8);
}
#endif
void code19(){
	//ADD HL,DE
	setT11_10();
	z80ADD16(regDE);
}
void code1A(){
	//LD A,(DE)
	setT7();
	loadA(readMemory(regDE));
}
void code1B(){
	//DEC DE
	setT6_5();
	loadDE(regDE-1);
}
void code1C(){
	UINT8 i8;
	//INC E
	setT4_5();
	i8=regE+1;
	loadE(i8);
	flag8inc(i8);
}
void code1D(){
	UINT8 i8;
	//DEC E
	setT4_5();
	i8=regE-1;
	loadE(i8);
	flag8dec(i8);
}
void code1E(){
	//LD E,n
	setT7();
	loadE(getCode());
}
void code1F(){
	UINT8 i8;
	//RRA C-> A7->A0 ->C
	setT4();
	i8=flagC;
	if (regA&0x01) setCflag();
	else clearCflag();		
	if (i8) loadA((regA>>1) | 0x80);
	else loadA(regA>>1);
	clearHflag();
	clearNflag();	
}
#ifdef Z80
void code20(){
	UINT8 i8;
	//JR NZ,(PC+e)
	if (flagZ) {
		setT7();
		getCode();
	} else {
		setT12();
		z80JR(i8);
	}
}
#endif
void code21(){
	//LD HL,nn
	setT10();
	loadL(getCode());
	loadH(getCode());
}
void code22(){
	UINT8 i8;
	UINT16 i16;
	//LD (nn),HL
	setT16();
	i8=getCode();
	i16=getCode()<<8;
	i16=i16 | i8;
	writeMemory(i16,regL);
	writeMemory(i16+1,regH);
}
void code23(){
	//INC HL
	setT6_5();
	loadHL(regHL+1);
}
void code24(){
	UINT8 i8;
	//INC H
	setT4_5();
	i8=regH+1;
	loadH(i8);
	flag8inc(i8);
}
void code25(){
	UINT8 i8;
	//DEC H
	setT4_5();
	i8=regH-1;
	loadH(i8);
	flag8dec(i8);
}
void code26(){
	//LD H,n
	setT7();
	loadH(getCode());
}
void code27(){
	UINT8 i8;
	UINT8 h;
	//DAA
	//This code is faster than one using DAA table.
	h=regA;
	setT4();
	if (!flagN) {
		if (!flagC) {
			if (!flagH) {
				if ((regA&0xF0)<0x90) {
					if ((regA&0x0F)<0x0A) {
						//loadA(regA+0x00);
						//clearCflag();
					} else {
						loadA(regA+0x06);
						//clearCflag();
					}
				} else if ((regA&0xF0)==0x90) {
					if ((regA&0x0F)<0x0A) {
						//loadA(regA+0x00);
						//clearCflag();
					} else {
						loadA(regA+0x66);
						setCflag();
					}
				} else {
					if ((regA&0x0F)<0x0A) {
						loadA(regA+0x60);
						setCflag();
					} else {
						loadA(regA+0x66);
						setCflag();
					}
				}
			} else {
				if ((regA&0xF0)<0x90) {
						loadA(regA+0x06);
						//clearCflag();
				} else if ((regA&0xF0)==0x90) {
					if ((regA&0x0F)<0x0A) {
						loadA(regA+0x06);
						//clearCflag();
					} else {
						loadA(regA+0x66);
						setCflag();
					}
				} else {
						loadA(regA+0x66);
						setCflag();
				}
			}
		} else {
			if (!flagH) {
					if ((regA&0x0F)<0x0A) {
						loadA(regA+0x60);
						//setCflag();
					} else {
						loadA(regA+0x66);
						//setCflag();
					}
			} else {
						loadA(regA+0x66);
						//setCflag();
			}
		}
	} else {
		if (!flagC) {
			if (!flagH) {
				if ((regA&0xF0)<0x90) {
					if ((regA&0x0F)<0x0A) {
						//loadA(regA+0x00);
						//clearCflag();
					} else {
						loadA(regA+0xFA);
						//clearCflag();
					}
				} else if ((regA&0xF0)==0x90) {
					if ((regA&0x0F)<0x0A) {
						//loadA(regA+0x00);
						//clearCflag();
					} else {
						loadA(regA+0x9A);
						setCflag();
					}
				} else {
					if ((regA&0x0F)<0x0A) {
						loadA(regA+0xA0);
						setCflag();
					} else {
						loadA(regA+0x9A);
						setCflag();
					}
				}
			} else {
				if ((regA&0xF0)<0x90) {
						loadA(regA+0xFA);
						//clearCflag();
				} else if ((regA&0xF0)==0x90) {
					if ((regA&0x0F)<0x0A) {
						loadA(regA+0xFA);
						//clearCflag();
					} else {
						loadA(regA+0x9A);
						setCflag();
					}
				} else {
						loadA(regA+0x9A);
						setCflag();
				}
			}
		} else {
			if (!flagH) {
					if ((regA&0x0F)<0x0A) {
						loadA(regA+0xA0);
						//setCflag();
					} else {
						loadA(regA+0x9A);
						//setCflag();
					}
			} else {
						loadA(regA+0x9A);
						//setCflag();
			}
		}
	}
	if (regA&0x80) setSflag();
	else clearSflag();
	if (regA) clearZflag();
	else setZflag();
	setZ80Parity(regA,i8);
	if ((h^regA)&0x10) setHflag();
	else clearHflag();
}
#ifdef Z80
void code28(){
	UINT8 i8;
	//JR Z,(PC+e)
	if (flagZ) {
		setT12();
		z80JR(i8);
	} else {
		setT7();
		getCode();
	}
}
#endif
void code29(){
	//ADD HL,HL
	setT11_10();
	z80ADD16(regHL);
}
void code2A(){
	UINT8 i8;
	UINT16 i16;
	//LD HL,(nn)
	setT16();
	i8=getCode();
	i16=getCode()<<8;
	i16|=i8;
	loadL(readMemory(i16));
	loadH(readMemory(i16+1));
}
void code2B(){
	//DEC HL
	setT6_5();
	loadHL(regHL-1);
}
void code2C(){
	UINT8 i8;
	//INC L
	setT4_5();
	i8=regL+1;
	loadL(i8);
	flag8inc(i8);
}
void code2D(){
	UINT8 i8;
	//DEC L
	setT4_5();
	i8=regL-1;
	loadL(i8);
	flag8dec(i8);
}
void code2E(){
	//LD L,n
	setT7();
	loadL(getCode());
}
void code2F(){
	//CPL
	setT4();
	loadA(~regA);
	setHflag();
	setNflag();
}
#ifdef Z80
void code30(){
	UINT8 i8;
	//JR NC,(PC+e)
	if (flagC) {
		setT7();
		getCode();
	} else {
		setT12();
		z80JR(i8);
	}
}
#endif
void code31(){
	UINT8 i8;
	UINT16 i16;
	//LD SP,nn
	setT10();
	i8=getCode();
	i16=getCode()<<8;
	i16=i16 | i8;
	loadSP(i16);
}
void code32(){
	UINT8 i8;
	UINT16 i16;
	//LD (nn),A
	setT13();
	i8=getCode();
	i16=getCode()<<8;
	i16=i16 | i8;
	writeMemory(i16,regA);
}
void code33(){
	//INC SP
	setT6_5();
	loadSP(regSP+1);
}
void code34(){
	UINT8 i8;
	//INC (HL)
	setT11_10();
	i8=readMemory(regHL)+1;
	writeMemory(regHL,i8);
	flag8inc(i8);
}
void code35(){
	UINT8 i8;
	//DEC (HL)
	setT11_10();
	i8=readMemory(regHL)-1;
	writeMemory(regHL,i8);
	flag8dec(i8);
}
void code36(){
	//LD (HL),n
	setT10();
	writeMemory(regHL,getCode());
}
void code37(){
	//SCF
	setT4();
	clearHflag();
	clearNflag();
	setCflag();
}
#ifdef Z80
void code38(){
	UINT8 i8;
	//JR C,(PC+e)
	if (flagC) {
		setT12();
		z80JR(i8);
	} else {
		setT7();
		getCode();
	}
}
#endif
void code39(){
	//ADD HL,SP
	setT11_10();
	z80ADD16(regSP);
}
void code3A(){
	UINT8 i8;
	UINT16 i16;
	//LD A,(nn)
	setT13();
	i8=getCode();
	i16=getCode()<<8;
	i16|=i8;
	loadA(readMemory(i16));
}
void code3B(){
	//DEC SP
	setT6_5();
	loadSP(regSP-1);
}
void code3C(){
	UINT8 i8;
	//INC A
	setT4_5();
	i8=regA+1;
	loadA(i8);
	flag8inc(i8);
}
void code3D(){
	UINT8 i8;
	//DEC A
	setT4_5();
	i8=regA-1;
	loadA(i8);
	flag8dec(i8);
}
void code3E(){
	//LD A,n
	setT7();
	loadA(getCode());
}
void code3F(){
	//CCF
	setT4();
	if (flagC) {
		setHflag();
		clearCflag();
	} else {
		clearHflag();
		setCflag();
	}
	clearNflag();
}
void code40(){
	//LD B,B
	setT4_5();
	loadB(regB);
}
void code41(){
	//LD B,C
	setT4_5();
	loadB(regC);
}
void code42(){
	//LD B,D
	setT4_5();
	loadB(regD);
}
void code43(){
	//LD B,E
	setT4_5();
	loadB(regE);
}
void code44(){
	//LD B,H
	setT4_5();
	loadB(regH);
}
void code45(){
	//LD B,L
	setT4_5();
	loadB(regL);
}
void code46(){
	//LD B,(HL)
	setT7();
	loadB(readMemory(regHL));
}
void code47(){
	//LD B,A
	setT4_5();
	loadB(regA);
}
void code48(){
	//LD C,B
	setT4_5();
	loadC(regB);
}
void code49(){
	//LD C,C
	setT4_5();
	loadC(regC);
}
void code4A(){
	//LD C,D
	setT4_5();
	loadC(regD);
}
void code4B(){
	//LD C,E
	setT4_5();
	loadC(regE);
}
void code4C(){
	//LD C,H
	setT4_5();
	loadC(regH);
}
void code4D(){
	//LD C,L
	setT4_5();
	loadC(regL);
}
void code4E(){
	//LD C,(HL)
	setT7();
	loadC(readMemory(regHL));
}
void code4F(){
	//LD C,A
	setT4_5();
	loadC(regA);
}
void code50(){
	//LD D,B
	setT4_5();
	loadD(regB);
}
void code51(){
	//LD D,C
	setT4_5();
	loadD(regC);
}
void code52(){
	//LD D,D
	setT4_5();
	loadD(regD);
}
void code53(){
	//LD D,E
	setT4_5();
	loadD(regE);
}
void code54(){
	//LD D,H
	setT4_5();
	loadD(regH);
}
void code55(){
	//LD D,L
	setT4_5();
	loadD(regL);
}
void code56(){
	//LD D,(HL)
	setT7();
	loadD(readMemory(regHL));
}
void code57(){
	//LD D,A
	setT4_5();
	loadD(regA);
}
void code58(){
	//LD E,B
	setT4_5();
	loadE(regB);
}
void code59(){
	//LD E,C
	setT4_5();
	loadE(regC);
}
void code5A(){
	//LD E,D
	setT4_5();
	loadE(regD);
}
void code5B(){
	//LD E,E
	setT4_5();
	loadE(regE);
}
void code5C(){
	//LD E,H
	setT4_5();
	loadE(regH);
}
void code5D(){
	//LD E,L
	setT4_5();
	loadE(regL);
}
void code5E(){
	//LD E,(HL)
	setT7();
	loadE(readMemory(regHL));
}
void code5F(){
	//LD E,A
	setT4_5();
	loadE(regA);
}
void code60(){
	//LD H,B
	setT4_5();
	loadH(regB);
}
void code61(){
	//LD H,C
	setT4_5();
	loadH(regC);
}
void code62(){
	//LD H,D
	setT4_5();
	loadH(regD);
}
void code63(){
	//LD H,E
	setT4_5();
	loadH(regE);
}
void code64(){
	//LD H,H
	setT4_5();
	loadH(regH);
}
void code65(){
	//LD H,L
	setT4_5();
	loadH(regL);
}
void code66(){
	//LD H,(HL)
	setT7();
	loadH(readMemory(regHL));
}
void code67(){
	//LD H,A
	setT4_5();
	loadH(regA);
}
void code68(){
	//LD L,B
	setT4_5();
	loadL(regB);
}
void code69(){
	//LD L,C
	setT4_5();
	loadL(regC);
}
void code6A(){
	//LD L,D
	setT4_5();
	loadL(regD);
}
void code6B(){
	//LD L,E
	setT4_5();
	loadL(regE);
}
void code6C(){
	//LD L,H
	setT4_5();
	loadL(regH);
}
void code6D(){
	//LD L,L
	setT4_5();
	loadL(regL);
}
void code6E(){
	//LD L,(HL)
	setT7();
	loadL(readMemory(regHL));
}
void code6F(){
	//LD L,A
	setT4_5();
	loadL(regA);
}
void code70(){
	//LD (HL),B
	setT7();
	writeMemory(regHL,regB);
}
void code71(){
	//LD (HL),C
	setT7();
	writeMemory(regHL,regC);
}
void code72(){
	//LD (HL),D
	setT7();
	writeMemory(regHL,regD);
}
void code73(){
	//LD (HL),E
	setT7();
	writeMemory(regHL,regE);
}
void code74(){
	//LD (HL),H
	setT7();
	writeMemory(regHL,regH);
}
void code75(){
	//LD (HL),L
	setT7();
	writeMemory(regHL,regL);
}
void code76(){
	//HALT
	setT4_7();
	loadPC(regPC-1);
}
void code77(){
	//LD (HL),A
	setT7();
	writeMemory(regHL,regA);
}
void code78(){
	//LD A,B
	setT4_5();
	loadA(regB);
}
void code79(){
	//LD A,C
	setT4_5();
	loadA(regC);
}
void code7A(){
	//LD A,D
	setT4_5();
	loadA(regD);
}
void code7B(){
	//LD A,E
	setT4_5();
	loadA(regE);
}
void code7C(){
	//LD A,H
	setT4_5();
	loadA(regH);
}
void code7D(){
	//LD A,L
	setT4_5();
	loadA(regL);
}
void code7E(){
	//LD A,(HL)
	setT7();
	loadA(readMemory(regHL));
}
void code7F(){
	//LD A,A
	setT4_5();
	loadA(regA);
}
void code80(){
	//ADD A,B
	setT4();
	z80ADD(regB);
}
void code81(){
	//ADD A,C
	setT4();
	z80ADD(regC);
}
void code82(){
	//ADD A,D
	setT4();
	z80ADD(regD);
}
void code83(){
	//ADD A,E
	setT4();
	z80ADD(regE);
}
void code84(){
	//ADD A,H
	setT4();
	z80ADD(regH);
}
void code85(){
	//ADD A,L
	setT4();
	z80ADD(regL);
}
void code86(){
	//ADD A,(HL)
	setT7();
	z80ADD(readMemory(regHL));
}
void code87(){
	//ADD A,A
	setT4();
	z80ADD(regA);
}
void code88(){
	//ADC A,B
	setT4();
	z80ADC(regB);
}
void code89(){
	//ADC A,C
	setT4();
	z80ADC(regC);
}
void code8A(){
	//ADC A,D
	setT4();
	z80ADC(regD);
}
void code8B(){
	//ADC A,E
	setT4();
	z80ADC(regE);
}
void code8C(){
	//ADC A,H
	setT4();
	z80ADC(regH);
}
void code8D(){
	//ADC A,L
	setT4();
	z80ADC(regL);
}
void code8E(){
	//ADC A,(HL)
	setT7();
	z80ADC(readMemory(regHL));
}
void code8F(){
	//ADC A,A
	setT4();
	z80ADC(regA);
}
void code90(){
	//SUB B
	setT4();
	z80SUB(regB);
}
void code91(){
	//SUB C
	setT4();
	z80SUB(regC);
}
void code92(){
	//SUB D
	setT4();
	z80SUB(regD);
}
void code93(){
	//SUB E
	setT4();
	z80SUB(regE);
}
void code94(){
	//SUB H
	setT4();
	z80SUB(regH);
}
void code95(){
	//SUB L
	setT4();
	z80SUB(regL);
}
void code96(){
	//SUB (HL)
	setT7();
	z80SUB(readMemory(regHL));
}
void code97(){
	//SUB A
	setT4();
	z80SUB(regA);
}
void code98(){
	//SBC A,B
	setT4();
	z80SBC(regB);
}
void code99(){
	//SBC A,C
	setT4();
	z80SBC(regC);
}
void code9A(){
	//SBC A,D
	setT4();
	z80SBC(regD);
}
void code9B(){
	//SBC A,E
	setT4();
	z80SBC(regE);
}
void code9C(){
	//SBC A,H
	setT4();
	z80SBC(regH);
}
void code9D(){
	//SBC A,L
	setT4();
	z80SBC(regL);
}
void code9E(){
	//SBC A,(HL)
	setT7();
	z80SBC(readMemory(regHL));
}
void code9F(){
	//SBC A,A
	setT4();
	z80SBC(regA);
}
void codeA0(){
	//AND B
	setT4();
	z80AND(regB);
}
void codeA1(){
	//AND C
	setT4();
	z80AND(regC);
}
void codeA2(){
	//AND D
	setT4();
	z80AND(regD);
}
void codeA3(){
	//AND E
	setT4();
	z80AND(regE);
}
void codeA4(){
	//AND H
	setT4();
	z80AND(regH);
}
void codeA5(){
	//AND L
	setT4();
	z80AND(regL);
}
void codeA6(){
	//AND (HL)
	setT7();
	z80AND(readMemory(regHL));
}
void codeA7(){
	//AND A
	setT4();
	z80AND(regA);
}
void codeA8(){
	//XOR B
	setT4();
	z80XOR(regB);
}
void codeA9(){
	//XOR C
	setT4();
	z80XOR(regC);
}
void codeAA(){
	//XOR D
	setT4();
	z80XOR(regD);
}
void codeAB(){
	//XOR E
	setT4();
	z80XOR(regE);
}
void codeAC(){
	//XOR H
	setT4();
	z80XOR(regH);
}
void codeAD(){
	//XOR L
	setT4();
	z80XOR(regL);
}
void codeAE(){
	//XOR (HL)
	setT7();
	z80XOR(readMemory(regHL));
}
void codeAF(){
	//XOR A
	setT4();
	z80XOR(regA);
}
void codeB0(){
	//OR B
	setT4();
	z80OR(regB);
}
void codeB1(){
	//OR C
	setT4();
	z80OR(regC);
}
void codeB2(){
	//OR D
	setT4();
	z80OR(regD);
}
void codeB3(){
	//OR E
	setT4();
	z80OR(regE);
}
void codeB4(){
	//OR H
	setT4();
	z80OR(regH);
}
void codeB5(){
	//OR L
	setT4();
	z80OR(regL);
}
void codeB6(){
	//OR (HL)
	setT7();
	z80OR(readMemory(regHL));
}
void codeB7(){
	//OR A
	setT4();
	z80OR(regA);
}
void codeB8(){
	//CP B
	setT4();
	z80CP(regB);
}
void codeB9(){
	//CP C
	setT4();
	z80CP(regC);
}
void codeBA(){
	//CP D
	setT4();
	z80CP(regD);
}
void codeBB(){
	//CP E
	setT4();
	z80CP(regE);
}
void codeBC(){
	//CP H
	setT4();
	z80CP(regH);
}
void codeBD(){
	//CP L
	setT4();
	z80CP(regL);
}
void codeBE(){
	//CP (HL)
	setT7();
	z80CP(readMemory(regHL));
}
void codeBF(){
	//CP A
	setT4();
	z80CP(regA);
}
void codeC0(){
	UINT8 i8;
	UINT16 i16;
	//RET NZ
	if (flagZ) {
		setT5();
	} else {
		setT11();
		i8=readMemory(regSP);
		i16=readMemory(regSP+1)<<8;
		i16|=i8;
		loadPC(i16);
		loadSP(regSP+2);
	}
}
void codeC1(){
	//POP BC
	setT10();
	loadC(readMemory(regSP));
	loadB(readMemory(regSP+1));
	loadSP(regSP+2);
}
void codeC2(){
	UINT8 i8;
	UINT16 i16;
	//JP NZ,nn
	setT10();
	i8=getCode();
	i16=getCode();
	if (!flagZ) loadPC( (i16<<8) | i8);
}
void codeC3(){
	UINT8 i8;
	UINT16 i16;
	//JP nn
	setT10();
	i8=getCode();
	i16=getCode();
	loadPC( (i16<<8) | i8);
}
void codeC4(){
	UINT8 i8;
	UINT16 i16;
	//CALL NZ,nn
	i8=getCode();
	i16=getCode();
	if (flagZ){
		setT10_11();
	} else {
		setT17();
		loadSP(regSP-2);
		writeMemory(regSP,regPC&0xFF);
		writeMemory(regSP+1,regPC>>8);
		loadPC((i16<<8)|i8);
	}
}
void codeC5(){
	//PUSH BC
	setT11();
	loadSP(regSP-2);
	writeMemory(regSP,regC);
	writeMemory(regSP+1,regB);
}
void codeC6(){
	//ADD A,n
	setT7();
	z80ADD(getCode());
}
void codeC7(){
	//RST 0H
	setT11();
	loadSP(regSP-2);
	writeMemory(regSP,regPC&0xFF);
	writeMemory(regSP+1,regPC>>8);
	loadPC(0x00);
}
void codeC8(){
	UINT8 i8;
	UINT16 i16;
	//RET Z
	if (flagZ) {
		setT11();
		i8=readMemory(regSP);
		i16=readMemory(regSP+1)<<8;
		i16|=i8;
		loadPC(i16);
		loadSP(regSP+2);
	} else {
		setT5();
	}
}
void codeC9(){
	UINT8 i8;
	UINT16 i16;
	//RET
	setT10();
	i8=readMemory(regSP);
	i16=readMemory(regSP+1)<<8;
	i16|=i8;
	loadPC(i16);
	loadSP(regSP+2);
}
void codeCA(){
	UINT8 i8;
	UINT16 i16;
	//JP Z,nn
	setT10();
	i8=getCode();
	i16=getCode();
	if (flagZ) loadPC( (i16<<8) | i8);
}
void codeCC(){
	UINT8 i8;
	UINT16 i16;
	//CALL Z,nn
	i8=getCode();
	i16=getCode();
	if (flagZ){
		setT17();
		loadSP(regSP-2);
		writeMemory(regSP,regPC&0xFF);
		writeMemory(regSP+1,regPC>>8);
		loadPC((i16<<8)|i8);
	} else {
		setT10_11();
	}
}
void codeCD(){
	UINT8 i8;
	UINT16 i16;
	//CALL nn
	setT17();
	i8=getCode();
	i16=getCode();
	loadSP(regSP-2);
	writeMemory(regSP,regPC&0xFF);
	writeMemory(regSP+1,regPC>>8);
	loadPC((i16<<8)|i8);
}
void codeCE(){
	//ADC A,n
	setT7();
	z80ADC(getCode());
}
void codeCF(){
	//RST 8H
	setT11();
	loadSP(regSP-2);
	writeMemory(regSP,regPC&0xFF);
	writeMemory(regSP+1,regPC>>8);
	loadPC(0x08);
}
void codeD0(){
	UINT8 i8;
	UINT16 i16;
	//RET NC
	if (flagC) {
		setT5();
	} else {
		setT11();
		i8=readMemory(regSP);
		i16=readMemory(regSP+1)<<8;
		i16|=i8;
		loadPC(i16);
		loadSP(regSP+2);
	}
}
void codeD1(){
	//POP DE
	setT10();
	loadE(readMemory(regSP));
	loadD(readMemory(regSP+1));
	loadSP(regSP+2);
}
void codeD2(){
	UINT8 i8;
	UINT16 i16;
	//JP NC,nn
	setT10();
	i8=getCode();
	i16=getCode();
	if (!flagC) loadPC( (i16<<8) | i8);
}
void codeD3(){
	//OUT (n),A
	setT11_10();
	writeIO(getCode(),regA,regA);
}
void codeD4(){
	UINT8 i8;
	UINT16 i16;
	//CALL NC,nn
	i8=getCode();
	i16=getCode();
	if (flagC){
		setT10_11();
	} else {
		setT17();
		loadSP(regSP-2);
		writeMemory(regSP,regPC&0xFF);
		writeMemory(regSP+1,regPC>>8);
		loadPC((i16<<8)|i8);
	}
}
void codeD5(){
	//PUSH DE
	setT11();
	loadSP(regSP-2);
	writeMemory(regSP,regE);
	writeMemory(regSP+1,regD);
}
void codeD6(){
	//SUB n
	setT7();
	z80SUB(getCode());
}
void codeD7(){
	//RST 10H
	setT11();
	loadSP(regSP-2);
	writeMemory(regSP,regPC&0xFF);
	writeMemory(regSP+1,regPC>>8);
	loadPC(0x10);
}
void codeD8(){
	UINT8 i8;
	UINT16 i16;
	//RET C
	if (flagC) {
		setT11();
		i8=readMemory(regSP);
		i16=readMemory(regSP+1)<<8;
		i16|=i8;
		loadPC(i16);
		loadSP(regSP+2);
	} else {
		setT5();
	}
}
#ifdef Z80
void codeD9(){
	UINT16 i16;
	//EXX
	setT4();
	i16=regBC;
	loadBC(regBCprime);
	loadBCprime(i16);
	i16=regDE;
	loadDE(regDEprime);
	loadDEprime(i16);
	i16=regHL;
	loadHL(regHLprime);
	loadHLprime(i16);
}
#endif
void codeDA(){
	UINT8 i8;
	UINT16 i16;
	//JP C,nn
	setT10();
	i8=getCode();
	i16=getCode();
	if (flagC) loadPC( (i16<<8) | i8);
}
void codeDB(){
	//IN A,n
	setT11_10();
	loadA(readIO(getCode(), regA));
}
void codeDC(){
	UINT8 i8;
	UINT16 i16;
	//CALL C,nn
	i8=getCode();
	i16=getCode();
	if (flagC){
		setT17();
		loadSP(regSP-2);
		writeMemory(regSP,regPC&0xFF);
		writeMemory(regSP+1,regPC>>8);
		loadPC((i16<<8)|i8);
	} else {
		setT10_11();
	}
}
void codeDE(){
	//SBC   A,n
	setT7();
	z80SBC(getCode());
}
void codeDF(){
	//RST   18H
	setT11();
	loadSP(regSP-2);
	writeMemory(regSP,regPC&0xFF);
	writeMemory(regSP+1,regPC>>8);
	loadPC(0x18);
}
void codeE0(){
	UINT8 i8;
	UINT16 i16;
	//RET   PO
	if (flagP) {
		setT5();
	} else {
		setT11();
		i8=readMemory(regSP);
		i16=readMemory(regSP+1)<<8;
		i16|=i8;
		loadPC(i16);
		loadSP(regSP+2);
	}
}
void codeE1(){
	//POP   HL
	setT10();
	loadL(readMemory(regSP));
	loadH(readMemory(regSP+1));
	loadSP(regSP+2);
}
void codeE2(){
	UINT8 i8;
	UINT16 i16;
	//E2   JP    PO,nn
	setT10();
	i8=getCode();
	i16=getCode();
	if (!flagP) loadPC( (i16<<8) | i8);
}
void codeE3(){
	UINT16 i16;
	//EX    (SP),HL
	setT19_18();
	i16=readMemory(regSP+1)<<8;
	i16|=readMemory(regSP);
	writeMemory(regSP,regL);
	writeMemory(regSP+1,regH);
	loadHL(i16);
}
void codeE4(){
	UINT8 i8;
	UINT16 i16;
	//CALL  PO,nn
	i8=getCode();
	i16=getCode();
	if (flagP){
		setT10_11();
	} else {
		setT17();
		loadSP(regSP-2);
		writeMemory(regSP,regPC&0xFF);
		writeMemory(regSP+1,regPC>>8);
		loadPC((i16<<8)|i8);
	}
}
void codeE5(){
	//PUSH  HL
	setT11();
	loadSP(regSP-2);
	writeMemory(regSP,regL);
	writeMemory(regSP+1,regH);
}
void codeE6(){
	//AND   n
	setT7();
	z80AND(getCode());
}
void codeE7(){
	//RST   20H
	setT11();
	loadSP(regSP-2);
	writeMemory(regSP,regPC&0xFF);
	writeMemory(regSP+1,regPC>>8);
	loadPC(0x20);
}
void codeE8(){
	UINT8 i8;
	UINT16 i16;
	//RET   PE
	if (flagP) {
		setT11();
		i8=readMemory(regSP);
		i16=readMemory(regSP+1)<<8;
		i16|=i8;
		loadPC(i16);
		loadSP(regSP+2);
	} else {
		setT5();
	}
}
void codeE9(){
	//JP    (HL)
	setT4_5();
	loadPC(regHL);
}
void codeEA(){
	UINT8 i8;
	UINT16 i16;
	//JP    PE,nn
	setT10();
	i8=getCode();
	i16=getCode();
	if (flagP) loadPC( (i16<<8) | i8);
}
void codeEB(){
	UINT16 i16;
	//EX    DE,HL
	setT4();
	i16=regHL;
	loadHL(regDE);
	loadDE(i16);
}
void codeEC(){
	UINT8 i8;
	UINT16 i16;
	//CALL  PE,NN
	i8=getCode();
	i16=getCode();
	if (flagP){
		setT17();
		loadSP(regSP-2);
		writeMemory(regSP,regPC&0xFF);
		writeMemory(regSP+1,regPC>>8);
		loadPC((i16<<8)|i8);
	} else {
		setT10_11();
	}
}
void codeEE(){
	//XOR   n
	setT7();
	z80XOR(getCode());
}
void codeEF(){
	//RST   28H
	setT11();
	loadSP(regSP-2);
	writeMemory(regSP,regPC&0xFF);
	writeMemory(regSP+1,regPC>>8);
	loadPC(0x28);
}
void codeF0(){
	UINT8 i8;
	UINT16 i16;
	//RET   P
	if (flagS) {
		setT5();
	} else {
		setT11();
		i8=readMemory(regSP);
		i16=readMemory(regSP+1)<<8;
		i16|=i8;
		loadPC(i16);
		loadSP(regSP+2);
	}
}
void codeF1(){
	//POP   AF
	setT10();
	loadF(readMemory(regSP));
	loadA(readMemory(regSP+1));
	loadSP(regSP+2);
}
void codeF2(){
	UINT8 i8;
	UINT16 i16;
	//F2   JP    P,nn
	setT10();
	setT10();
	i8=getCode();
	i16=getCode();
	if (!flagS) loadPC( (i16<<8) | i8);
}
void codeF3(){
	//DI
	setT4();
	loadIFF1(0);
	loadIFF2(0);
}
void codeF4(){
	UINT8 i8;
	UINT16 i16;
	//CALL  P,nn
	i8=getCode();
	i16=getCode();
	if (flagS){
		setT10_11();
	} else {
		setT17();
		loadSP(regSP-2);
		writeMemory(regSP,regPC&0xFF);
		writeMemory(regSP+1,regPC>>8);
		loadPC((i16<<8)|i8);
	}
}
void codeF5(){
	//PUSH  AF
	setT11();
	loadSP(regSP-2);
	writeMemory(regSP,regF);
	writeMemory(regSP+1,regA);
}
void codeF6(){
	//F6   OR    n
	setT7();
	z80OR(getCode());
}
void codeF7(){
	//RST   30H
	setT11();
	loadSP(regSP-2);
	writeMemory(regSP,regPC&0xFF);
	writeMemory(regSP+1,regPC>>8);
	loadPC(0x30);
}
void codeF8(){
	UINT8 i8;
	UINT16 i16;
	//RET   M
	if (flagS) {
		setT11();
		i8=readMemory(regSP);
		i16=readMemory(regSP+1)<<8;
		i16|=i8;
		loadPC(i16);
		loadSP(regSP+2);
	} else {
		setT5();
	}
}
void codeF9(){
	//LD    SP,HL
	setT6_5();
	loadSP(regHL);
}
void codeFA(){
	UINT8 i8;
	UINT16 i16;
	//FA   JP    M,nn
	setT10();
	i8=getCode();
	i16=getCode();
	if (flagS) loadPC( (i16<<8) | i8);
}
void codeFB(){
	//EI
	setT4();
	loadIFF1(1);
	loadIFF2(1);
}
void codeFC(){
	UINT8 i8;
	UINT16 i16;
	//CALL  M,nn
	i8=getCode();
	i16=getCode();
	if (flagS){
		setT17();
		loadSP(regSP-2);
		writeMemory(regSP,regPC&0xFF);
		writeMemory(regSP+1,regPC>>8);
		loadPC((i16<<8)|i8);
	} else {
		setT10_11();
	}
}
void codeFE(){
	//FE   CP    n
	setT7();
	z80CP(getCode());
}
void codeFF(){
	//RST   38H
	setT11();
	loadSP(regSP-2);
	writeMemory(regSP,regPC&0xFF);
	writeMemory(regSP+1,regPC>>8);
	loadPC(0x38);
}
#ifdef Z80
void codeCB00(){
	//RLC   B
	setT8();
	loadB(z80RLC(regB));
}
void codeCB01(){
	//RLC   C
	setT8();
	loadC(z80RLC(regC));
}
void codeCB02(){
	//RLC   D
	setT8();
	loadD(z80RLC(regD));
}
void codeCB03(){
	//RLC   E
	setT8();
	loadE(z80RLC(regE));
}
void codeCB04(){
	//RLC   H
	setT8();
	loadH(z80RLC(regH));
}
void codeCB05(){
	//RLC   L
	setT8();
	loadL(z80RLC(regL));
}
void codeCB06(){
	//RLC   (HL)
	setT15();
	writeMemory(regHL,z80RLC(readMemory(regHL)));
}
void codeCB07(){
	//RLC   A
	setT8();
	loadA(z80RLC(regA));
}
void codeCB08(){
	//RRC   B
	setT8();
	loadB(z80RRC(regB));
}
void codeCB09(){
	//RRC   C
	setT8();
	loadC(z80RRC(regC));
}
void codeCB0A(){
	//CRRC   D
	setT8();
	loadD(z80RRC(regD));
}
void codeCB0B(){
	//RRC   E
	setT8();
	loadE(z80RRC(regE));
}
void codeCB0C(){
	//RRC   H
	setT8();
	loadH(z80RRC(regH));
}
void codeCB0D(){
	//RRC   L
	setT8();
	loadL(z80RRC(regL));
}
void codeCB0E(){
	//RRC   (HL)
	setT15();
	writeMemory(regHL,z80RRC(readMemory(regHL)));
}
void codeCB0F(){
	//RRC   A
	setT8();
	loadA(z80RRC(regA));
}
void codeCB10(){
	//RL    B
	setT8();
	loadB(z80RL(regB));
}
void codeCB11(){
	//RL    C
	setT8();
	loadC(z80RL(regC));
}
void codeCB12(){
	//RL    D
	setT8();
	loadD(z80RL(regD));
}
void codeCB13(){
	//RL    E
	setT8();
	loadE(z80RL(regE));
}
void codeCB14(){
	//RL    H
	setT8();
	loadH(z80RL(regH));
}
void codeCB15(){
	//RL    L
	setT8();
	loadL(z80RL(regL));
}
void codeCB16(){
	//RL    (HL)
	setT15();
	writeMemory(regHL,z80RL(readMemory(regHL)));
}
void codeCB17(){
	//RL    A
	setT8();
	loadA(z80RL(regA));
}
void codeCB18(){
	//CB 18   RR    B
	setT8();
	loadB(z80RR(regB));
}
void codeCB19(){
	//CB 19   RR    C
	setT8();
	loadC(z80RR(regC));
}
void codeCB1A(){
	//CB 1A   RR    D
	setT8();
	loadD(z80RR(regD));
}
void codeCB1B(){
	//CB 1B   RR    E
	setT8();
	loadE(z80RR(regE));
}
void codeCB1C(){
	//CB 1C   RR    H
	setT8();
	loadH(z80RR(regH));
}
void codeCB1D(){
	//CB 1D   RR    L
	setT8();
	loadL(z80RR(regL));
}
void codeCB1E(){
	//CB 1E   RR    (HL)
	setT15();
	writeMemory(regHL,z80RR(readMemory(regHL)));
}
void codeCB1F(){
	//CB 1F   RR    A
	setT8();
	loadA(z80RR(regA));
}
void codeCB20(){
	//SLA   B
	setT8();
	loadB(z80SLA(regB));
}
void codeCB21(){
	//SLA   C
	setT8();
	loadC(z80SLA(regC));
}
void codeCB22(){
	//SLA   D
	setT8();
	loadD(z80SLA(regD));
}
void codeCB23(){
	//SLA   E
	setT8();
	loadE(z80SLA(regE));
}
void codeCB24(){
	//SLA   H
	setT8();
	loadH(z80SLA(regH));
}
void codeCB25(){
	//SLA   L
	setT8();
	loadL(z80SLA(regL));
}
void codeCB26(){
	//SLA   (HL)
	setT15();
	writeMemory(regHL,z80SLA(readMemory(regHL)));
}
void codeCB27(){
	//SLA   A
	setT8();
	loadA(z80SLA(regA));
}
void codeCB28(){
	//SRA   B
	setT8();
	loadB(z80SRA(regB));
}
void codeCB29(){
	//SRA   C
	setT8();
	loadC(z80SRA(regC));
}
void codeCB2A(){
	//SRA   D
	setT8();
	loadD(z80SRA(regD));
}
void codeCB2B(){
	//SRA   E
	setT8();
	loadE(z80SRA(regE));
}
void codeCB2C(){
	//SRA   H
	setT8();
	loadH(z80SRA(regH));
}
void codeCB2D(){
	//SRA   L
	setT8();
	loadL(z80SRA(regL));
}
void codeCB2E(){
	//SRA   (HL)
	setT15();
	writeMemory(regHL,z80SRA(readMemory(regHL)));
}
void codeCB2F(){
	//SRA   A
	setT8();
	loadA(z80SRA(regA));
}
void codeCB38(){
	//SRL   B
	setT8();
	loadB(z80SRL(regB));
}
void codeCB39(){
	//SRL   C
	setT8();
	loadC(z80SRL(regC));
}
void codeCB3A(){
	//SRL   D
	setT8();
	loadD(z80SRL(regD));
}
void codeCB3B(){
	//SRL   E
	setT8();
	loadE(z80SRL(regE));
}
void codeCB3C(){
	//SRL   H
	setT8();
	loadH(z80SRL(regH));
}
void codeCB3D(){
	//SRL   L
	setT8();
	loadL(z80SRL(regL));
}
void codeCB3E(){
	//SRL   (HL)
	setT15();
	writeMemory(regHL,z80SRL(readMemory(regHL)));
}
void codeCB3F(){
	//SRL   A
	setT8();
	loadA(z80SRL(regA));
}
void codeCB40(){
	//BIT   0,B
	setT8();
	if (0x01&regB) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB41(){
	//BIT   0,C
	setT8();
	if (0x01&regC) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB42(){
	//BIT   0,D
	setT8();
	if (0x01&regD) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB43(){
	//BIT   0,E
	setT8();
	if (0x01&regE) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB44(){
	//BIT   0,H
	setT8();
	if (0x01&regH) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB45(){
	//BIT   0,L
	setT8();
	if (0x01&regL) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB46(){
	//BIT   0,(HL)
	setT12();
	if (0x01&readMemory(regHL)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB47(){
	//BIT   0,A
	setT8();
	if (0x01&regA) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB48(){
	//BIT   1,B
	setT8();
	if (0x02&regB) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB49(){
	//BIT   1,C
	setT8();
	if (0x02&regC) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB4A(){
	//BIT   1,D
	setT8();
	if (0x02&regD) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB4B(){
	//BIT   1,E
	setT8();
	if (0x02&regE) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB4C(){
	//BIT   1,H
	setT8();
	if (0x02&regH) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB4D(){
	//BIT   1,L
	setT8();
	if (0x02&regL) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB4E(){
	//BIT   1,(HL)
	setT12();
	if (0x02&readMemory(regHL)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB4F(){
	//BIT   1,A
	setT12();
	if (0x02&regA) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB50(){
	//BIT   2,B
	setT8();
	if (0x04&regB) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB51(){
	//BIT   2,C
	setT8();
	if (0x04&regC) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB52(){
	//BIT   2,D
	setT8();
	if (0x04&regD) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB53(){
	//BIT   2,E
	setT8();
	if (0x04&regE) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB54(){
	//BIT   2,H
	setT8();
	if (0x04&regH) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB55(){
	//BIT   2,L
	setT8();
	if (0x04&regL) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB56(){
	//BIT   2,(HL)
	setT12();
	if (0x04&readMemory(regHL)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB57(){
	//BIT   2,A
	setT8();
	if (0x04&regA) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB58(){
	//BIT   3,B
	setT8();
	if (0x08&regB) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB59(){
	//BIT   3,C
	setT8();
	if (0x08&regC) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB5A(){
	//BIT   3,D
	setT8();
	if (0x08&regD) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB5B(){
	//BIT   3,E
	setT8();
	if (0x08&regE) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB5C(){
	//BIT   3,H
	setT8();
	if (0x08&regH) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB5D(){
	//BIT   3,L
	setT8();
	if (0x08&regL) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB5E(){
	//BIT   3,(HL)
	setT12();
	if (0x08&readMemory(regHL)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB5F(){
	//BIT   3,A
	setT8();
	if (0x08&regA) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB60(){
	//BIT   4,B
	setT8();
	if (0x10&regB) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB61(){
	//BIT   4,C
	setT8();
	if (0x10&regC) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB62(){
	//BIT   4,D
	setT8();
	if (0x10&regD) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB63(){
	//BIT   4,E
	setT8();
	if (0x10&regE) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB64(){
	//BIT   4,H
	setT8();
	if (0x10&regH) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB65(){
	//BIT   4,L
	setT8();
	if (0x10&regL) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB66(){
	//BIT   4,(HL)
	setT12();
	if (0x10&readMemory(regHL)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB67(){
	//BIT   4,A
	setT8();
	if (0x10&regA) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB68(){
	//BIT   5,B
	setT8();
	if (0x20&regB) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB69(){
	//BIT   5,C
	setT8();
	if (0x20&regC) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB6A(){
	//BIT   5,D
	setT8();
	if (0x20&regD) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB6B(){
	//BIT   5,E
	setT8();
	if (0x20&regE) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB6C(){
	//BIT   5,H
	setT8();
	if (0x20&regH) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB6D(){
	//BIT   5,L
	setT8();
	if (0x20&regL) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB6E(){
	//BIT   5,(HL)
	setT12();
	if (0x20&readMemory(regHL)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB6F(){
	//BIT   5,A
	setT8();
	if (0x20&regA) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB70(){
	//BIT   6,B
	setT8();
	if (0x40&regB) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB71(){
	//BIT   6,C
	setT8();
	if (0x40&regC) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB72(){
	//BIT   6,D
	setT8();
	if (0x40&regD) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB73(){
	//BIT   6,E
	setT8();
	if (0x40&regE) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB74(){
	//BIT   6,H
	setT8();
	if (0x40&regH) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB75(){
	//BIT   6,L
	setT8();
	if (0x40&regL) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB76(){
	//BIT   6,(HL)
	setT12();
	if (0x40&readMemory(regHL)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB77(){
	//BIT   6,A
	setT8();
	if (0x40&regA) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB78(){
	//BIT   7,B
	setT8();
	if (0x80&regB) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB79(){
	//BIT   7,C
	setT8();
	if (0x80&regC) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB7A(){
	//BIT   7,D
	setT8();
	if (0x80&regD) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB7B(){
	//BIT   7,E
	setT8();
	if (0x80&regE) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB7C(){
	//BIT   7,H
	setT8();
	if (0x80&regH) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB7D(){
	//BIT   7,L
	setT8();
	if (0x80&regL) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB7E(){
	//BIT   7,(HL)
	setT12();
	if (0x80&readMemory(regHL)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB7F(){
	//BIT   7,A
	setT8();
	if (0x80&regA) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeCB80(){
	//RES   0,B
	setT8();
	loadB(0xFE&regB);
}
void codeCB81(){
	//RES   0,C
	setT8();
	loadC(0xFE&regC);
}
void codeCB82(){
	//RES   0,D
	setT8();
	loadD(0xFE&regD);
}
void codeCB83(){
	//RES   0,E
	setT8();
	loadE(0xFE&regE);
}
void codeCB84(){
	//RES   0,H
	setT8();
	loadH(0xFE&regH);
}
void codeCB85(){
	//RES   0,L
	setT8();
	loadL(0xFE&regL);
}
void codeCB86(){
	//RES   0,(HL)
	setT15();
	writeMemory(regHL,0xFE&readMemory(regHL));
}
void codeCB87(){
	//RES   0,A
	setT8();
	loadA(0xFE&regA);
}
void codeCB88(){
	//RES   1,B
	setT8();
	loadB(0xFD&regB);
}
void codeCB89(){
	//RES   1,C
	setT8();
	loadC(0xFD&regC);
}
void codeCB8A(){
	//RES   1,D
	setT8();
	loadD(0xFD&regD);
}
void codeCB8B(){
	//RES   1,E
	setT8();
	loadE(0xFD&regE);
}
void codeCB8C(){
	//RES   1,H
	setT8();
	loadH(0xFD&regH);
}
void codeCB8D(){
	//RES   1,L
	setT8();
	loadL(0xFD&regL);
}
void codeCB8E(){
	//RES   1,(HL)
	setT15();
	writeMemory(regHL,0xFD&readMemory(regHL));
}
void codeCB8F(){
	//RES   1,A
	setT8();
	loadA(0xFD&regA);
}
void codeCB90(){
	//RES   2,B
	setT8();
	loadB(0xFB&regB);
}
void codeCB91(){
	//RES   2,C
	setT8();
	loadC(0xFB&regC);
}
void codeCB92(){
	//RES   2,D
	setT8();
	loadD(0xFB&regD);
}
void codeCB93(){
	//RES   2,E
	setT8();
	loadE(0xFB&regE);
}
void codeCB94(){
	//RES   2,H
	setT8();
	loadH(0xFB&regH);
}
void codeCB95(){
	//RES   2,L
	setT8();
	loadL(0xFB&regL);
}
void codeCB96(){
	//RES   2,(HL)
	setT15();
	writeMemory(regHL,0xFB&readMemory(regHL));
}
void codeCB97(){
	//RES   2,A
	setT8();
	loadA(0xFB&regA);
}
void codeCB98(){
	//RES   3,B
	setT8();
	loadB(0xF7&regB);
}
void codeCB99(){
	//RES   3,C
	setT8();
	loadC(0xF7&regC);
}
void codeCB9A(){
	//RES   3,D
	setT8();
	loadD(0xF7&regD);
}
void codeCB9B(){
	//RES   3,E
	setT8();
	loadE(0xF7&regE);
}
void codeCB9C(){
	//RES   3,H
	setT8();
	loadH(0xF7&regH);
}
void codeCB9D(){
	//RES   3,L
	setT8();
	loadL(0xF7&regL);
}
void codeCB9E(){
	//RES   3,(HL)
	setT15();
	writeMemory(regHL,0xF7&readMemory(regHL));
}
void codeCB9F(){
	//RES   3,A
	setT8();
	loadA(0xF7&regA);
}
void codeCBA0(){
	//RES   4,B
	setT8();
	loadB(0xEF&regB);
}
void codeCBA1(){
	//RES   4,C
	setT8();
	loadC(0xEF&regC);
}
void codeCBA2(){
	//RES   4,D
	setT8();
	loadD(0xEF&regD);
}
void codeCBA3(){
	//RES   4,E
	setT8();
	loadE(0xEF&regE);
}
void codeCBA4(){
	//RES   4,H
	setT8();
	loadH(0xEF&regH);
}
void codeCBA5(){
	//RES   4,L
	setT8();
	loadL(0xEF&regL);
}
void codeCBA6(){
	//RES   4,(HL)
	setT15();
	writeMemory(regHL,0xEF&readMemory(regHL));
}
void codeCBA7(){
	//RES   4,A
	setT8();
	loadA(0xEF&regA);
}
void codeCBA8(){
	//RES   5,B
	setT8();
	loadB(0xDF&regB);
}
void codeCBA9(){
	//RES   5,C
	setT8();
	loadC(0xDF&regC);
}
void codeCBAA(){
	//RES   5,D
	setT8();
	loadD(0xDF&regD);
}
void codeCBAB(){
	//RES   5,E
	setT8();
	loadE(0xDF&regE);
}
void codeCBAC(){
	//RES   5,H
	setT8();
	loadH(0xDF&regH);
}
void codeCBAD(){
	//RES   5,L
	setT8();
	loadL(0xDF&regL);
}
void codeCBAE(){
	//RES   5,(HL)
	setT15();
	writeMemory(regHL,0xDF&readMemory(regHL));
}
void codeCBAF(){
	//RES   5,A
	setT8();
	loadA(0xDF&regA);
}
void codeCBB0(){
	//RES   6,B
	setT8();
	loadB(0xBF&regB);
}
void codeCBB1(){
	//RES   6,C
	setT8();
	loadC(0xBF&regC);
}
void codeCBB2(){
	//RES   6,D
	setT8();
	loadD(0xBF&regD);
}
void codeCBB3(){
	//RES   6,E
	setT8();
	loadE(0xBF&regE);
}
void codeCBB4(){
	//RES   6,H
	setT8();
	loadH(0xBF&regH);
}
void codeCBB5(){
	//RES   6,L
	setT8();
	loadL(0xBF&regL);
}
void codeCBB6(){
	//RES   6,(HL)
	setT15();
	writeMemory(regHL,0xBF&readMemory(regHL));
}
void codeCBB7(){
	//RES   6,A
	setT8();
	loadA(0xBF&regA);
}
void codeCBB8(){
	//RES   7,B
	setT8();
	loadB(0x7F&regB);
}
void codeCBB9(){
	//RES   7,C
	setT8();
	loadC(0x7F&regC);
}
void codeCBBA(){
	//RES   7,D
	setT8();
	loadD(0x7F&regD);
}
void codeCBBB(){
	//RES   7,E
	setT8();
	loadE(0x7F&regE);
}
void codeCBBC(){
	//RES   7,H
	setT8();
	loadH(0x7F&regH);
}
void codeCBBD(){
	//RES   7,L
	setT8();
	loadL(0x7F&regL);
}
void codeCBBE(){
	//RES   7,(HL)
	setT15();
	writeMemory(regHL,0x7F&readMemory(regHL));
}
void codeCBBF(){
	//RES   7,A
	setT8();
	loadA(0x7F&regA);
}
void codeCBC0(){
	//SET   0,B
	setT8();
	loadB(0x01|regB);
}
void codeCBC1(){
	//SET   0,C
	setT8();
	loadC(0x01|regC);
}
void codeCBC2(){
	//SET   0,D
	setT8();
	loadD(0x01|regD);
}
void codeCBC3(){
	//SET   0,E
	setT8();
	loadE(0x01|regE);
}
void codeCBC4(){
	//SET   0,H
	setT8();
	loadH(0x01|regH);
}
void codeCBC5(){
	//SET   0,L
	setT8();
	loadL(0x01|regL);
}
void codeCBC6(){
	//SET   0,(HL)
	setT15();
	writeMemory(regHL,0x01|readMemory(regHL));
}
void codeCBC7(){
	//SET   0,A
	setT8();
	loadA(0x01|regA);
}
void codeCBC8(){
	//SET   1,B
	setT8();
	loadB(0x02|regB);
}
void codeCBC9(){
	//SET   1,C
	setT8();
	loadC(0x02|regC);
}
void codeCBCA(){
	//SET   1,D
	setT8();
	loadD(0x02|regD);
}
void codeCBCB(){
	//SET   1,E
	setT8();
	loadE(0x02|regE);
}
void codeCBCC(){
	//SET   1,H
	setT8();
	loadH(0x02|regH);
}
void codeCBCD(){
	//SET   1,L
	setT8();
	loadL(0x02|regL);
}
void codeCBCE(){
	//SET   1,(HL)
	setT15();
	writeMemory(regHL,0x02|readMemory(regHL));
}
void codeCBCF(){
	//SET   1,A
	setT8();
	loadA(0x02|regA);
}
void codeCBD0(){
	//SET   2,B
	setT8();
	loadB(0x04|regB);
}
void codeCBD1(){
	//SET   2,C
	setT8();
	loadC(0x04|regC);
}
void codeCBD2(){
	//SET   2,D
	setT8();
	loadD(0x04|regD);
}
void codeCBD3(){
	//SET   2,E
	setT8();
	loadE(0x04|regE);
}
void codeCBD4(){
	//SET   2,H
	setT8();
	loadH(0x04|regH);
}
void codeCBD5(){
	//SET   2,L
	setT8();
	loadL(0x04|regL);
}
void codeCBD6(){
	//SET   2,(HL)
	setT15();
	writeMemory(regHL,0x04|readMemory(regHL));
}
void codeCBD7(){
	//SET   2,A
	setT8();
	loadA(0x04|regA);
}
void codeCBD8(){
	//SET   3,B
	setT8();
	loadB(0x08|regB);
}
void codeCBD9(){
	//SET   3,C
	setT8();
	loadC(0x08|regC);
}
void codeCBDA(){
	//SET   3,D
	setT8();
	loadD(0x08|regD);
}
void codeCBDB(){
	//SET   3,E
	setT8();
	loadE(0x08|regE);
}
void codeCBDC(){
	//SET   3,H
	setT8();
	loadH(0x08|regH);
}
void codeCBDD(){
	//SET   3,L
	setT8();
	loadL(0x08|regL);
}
void codeCBDE(){
	//SET   3,(HL)
	setT15();
	writeMemory(regHL,0x08|readMemory(regHL));
}
void codeCBDF(){
	//SET   3,A
	setT8();
	loadA(0x08|regA);
}
void codeCBE0(){
	//SET   4,B
	setT8();
	loadB(0x10|regB);
}
void codeCBE1(){
	//SET   4,C
	setT8();
	loadC(0x10|regC);
}
void codeCBE2(){
	//SET   4,D
	setT8();
	loadD(0x10|regD);
}
void codeCBE3(){
	//SET   4,E
	setT8();
	loadE(0x10|regE);
}
void codeCBE4(){
	//SET   4,H
	setT8();
	loadH(0x10|regH);
}
void codeCBE5(){
	//SET   4,L
	setT8();
	loadL(0x10|regL);
}
void codeCBE6(){
	//SET   4,(HL)
	setT15();
	writeMemory(regHL,0x10|readMemory(regHL));
}
void codeCBE7(){
	//SET   4,A
	setT8();
	loadA(0x10|regA);
}
void codeCBE8(){
	//SET   5,B
	setT8();
	loadB(0x20|regB);
}
void codeCBE9(){
	//SET   5,C
	setT8();
	loadC(0x20|regC);
}
void codeCBEA(){
	//SET   5,D
	setT8();
	loadD(0x20|regD);
}
void codeCBEB(){
	//SET   5,E
	setT8();
	loadE(0x20|regE);
}
void codeCBEC(){
	//SET   5,H
	setT8();
	loadH(0x20|regH);
}
void codeCBED(){
	//SET   5,L
	setT8();
	loadL(0x20|regL);
}
void codeCBEE(){
	//SET   5,(HL)
	setT15();
	writeMemory(regHL,0x20|readMemory(regHL));
}
void codeCBEF(){
	//SET   5,A
	setT8();
	loadA(0x20|regA);
}
void codeCBF0(){
	//SET   6,B
	setT8();
	loadB(0x40|regB);
}
void codeCBF1(){
	//SET   6,C
	setT8();
	loadC(0x40|regC);
}
void codeCBF2(){
	//SET   6,D
	setT8();
	loadD(0x40|regD);
}
void codeCBF3(){
	//SET   6,E
	setT8();
	loadE(0x40|regE);
}
void codeCBF4(){
	//SET   6,H
	setT8();
	loadH(0x40|regH);
}
void codeCBF5(){
	//SET   6,L
	setT8();
	loadL(0x40|regL);
}
void codeCBF6(){
	//SET   6,(HL)
	setT15();
	writeMemory(regHL,0x40|readMemory(regHL));
}
void codeCBF7(){
	//SET   6,A
	setT8();
	loadA(0x40|regA);
}
void codeCBF8(){
	//SET   7,B
	setT8();
	loadB(0x80|regB);
}
void codeCBF9(){
	//SET   7,C
	setT8();
	loadC(0x80|regC);
}
void codeCBFA(){
	//SET   7,D
	setT8();
	loadD(0x80|regD);
}
void codeCBFB(){
	//SET   7,E
	setT8();
	loadE(0x80|regE);
}
void codeCBFC(){
	//SET   7,H
	setT8();
	loadH(0x80|regH);
}
void codeCBFD(){
	//SET   7,L
	setT8();
	loadL(0x80|regL);
}
void codeCBFE(){
	//SET   7,(HL)
	setT15();
	writeMemory(regHL,0x80|readMemory(regHL));
}
void codeCBFF(){
	//SET   7,A
	setT8();
	loadA(0x80|regA);
}
void codeDD09(){
	UINT16 i16;
	UINT32 i32;
	//ADD   IX,BC
	setT15();
	i32=regIX+regBC;
	i16=(regIX&0x0FFF)+(regBC&0x0FFF);
	loadIX(i32);
	if (0xFFFF<i32) setCflag();
	else clearCflag();
	if (0x0FFF<i16) setHflag();
	else clearHflag();
	clearNflag();
}
void codeDD19(){
	UINT16 i16;
	UINT32 i32;
	//ADD   IX,DE
	setT15();
	i32=regIX+regDE;
	i16=(regIX&0x0FFF)+(regDE&0x0FFF);
	loadIX(i32);
	if (0xFFFF<i32) setCflag();
	else clearCflag();
	if (0x0FFF<i16) setHflag();
	else clearHflag();
	clearNflag();
}
void codeDD21(){
	UINT16 i16;
	//LD    IX,nn
	setT14();
	i16=getCode();
	i16|=getCode()<<8;
	loadIX(i16);
}
void codeDD22(){
	UINT16 i16;
	//LD    (nn),IX
	setT20();
	i16=getCode();
	i16|=getCode()<<8;
	writeMemory(i16,regIXl);
	writeMemory(i16+1,regIXh);
}
void codeDD23(){
	//INC   IX
	setT10();
	loadIX(regIX+1);
}
void codeDD29(){
	UINT16 i16;
	UINT32 i32;
	//ADD   IX,IX
	setT15();
	i32=regIX<<1;
	i16=(regIX&0x0FFF)<<1;
	loadIX(i32);
	if (0xFFFF<i32) setCflag();
	else clearCflag();
	if (0x0FFF<i16) setHflag();
	else clearHflag();
	clearNflag();
}
void codeDD2A(){
	UINT16 i16;
	//LD    IX,(nn)
	setT20();
	i16=getCode();
	i16|=getCode()<<8;
	loadIXl(readMemory(i16));
	loadIXh(readMemory(i16+1));
}
void codeDD2B(){
	//DEC   IX
	setT10();
	loadIX(regIX-1);
}
void codeDD34(){
	UINT8 i8;
	UINT16 i16;
	//INC   (IX+d)
	setT23();
	i16=regIX+getCodeIndex(i8);
	i8=readMemory(i16)+1;
	writeMemory(i16,i8);
	flag8inc(i8);
}
void codeDD35(){
	UINT8 i8;
	UINT16 i16;
	//DEC   (IX+D)
	setT23();
	i16=regIX+getCodeIndex(i8);
	i8=readMemory(i16)-1;
	writeMemory(i16,i8);
	flag8dec(i8);
}
void codeDD36(){
	UINT8 i8;
	UINT16 i16;
	//LD    (IX+d),n
	setT19();
	i16=regIX+getCodeIndex(i8);
	writeMemory(i16,getCode());
}
void codeDD39(){
	UINT16 i16;
	UINT32 i32;
	//ADD   IX,SP
	setT15();
	i32=regIX+regSP;
	i16=(regIX&0x0FFF)+(regSP&0x0FFF);
	loadIX(i32);
	if (0xFFFF<i32) setCflag();
	else clearCflag();
	if (0x0FFF<i16) setHflag();
	else clearHflag();
	clearNflag();
}
void codeDD46(){
	UINT8 i8;
	//LD    B,(IX+d)
	setT19();
	loadB(readMemory(regIX+getCodeIndex(i8)));
}
void codeDD4E(){
	UINT8 i8;
	//LD    C,(IX+d)
	setT19();
	loadC(readMemory(regIX+getCodeIndex(i8)));
}
void codeDD56(){
	UINT8 i8;
	//LD    D,(IX+d)
	setT19();
	loadD(readMemory(regIX+getCodeIndex(i8)));
}
void codeDD5E(){
	UINT8 i8;
	//LD    E,(IX+d)
	setT19();
	loadE(readMemory(regIX+getCodeIndex(i8)));
}
void codeDD66(){
	UINT8 i8;
	//LD    H,(IX+d)
	setT19();
	loadH(readMemory(regIX+getCodeIndex(i8)));
}
void codeDD6E(){
	UINT8 i8;
	//LD    L,(IX+d)
	setT19();
	loadL(readMemory(regIX+getCodeIndex(i8)));
}
void codeDD70(){
	UINT8 i8;
	//LD    (IX+d),B
	setT19();
	writeMemory(regIX+getCodeIndex(i8),regB);
}
void codeDD71(){
	UINT8 i8;
	//LD    (IX+d),C
	setT19();
	writeMemory(regIX+getCodeIndex(i8),regC);
}
void codeDD72(){
	UINT8 i8;
	//LD    (IX+d),D
	setT19();
	writeMemory(regIX+getCodeIndex(i8),regD);
}
void codeDD73(){
	UINT8 i8;
	//LD    (IX+d),E
	setT19();
	writeMemory(regIX+getCodeIndex(i8),regE);
}
void codeDD74(){
	UINT8 i8;
	//LD    (IX+d),H
	setT19();
	writeMemory(regIX+getCodeIndex(i8),regH);
}
void codeDD75(){
	UINT8 i8;
	//LD    (IX+d),L
	setT19();
	writeMemory(regIX+getCodeIndex(i8),regL);
}
void codeDD77(){
	UINT8 i8;
	//LD    (IX+d),A
	setT19();
	writeMemory(regIX+getCodeIndex(i8),regA);
}
void codeDD7E(){
	UINT8 i8;
	//LD    A,(IX+d)
	setT19();
	loadA(readMemory(regIX+getCodeIndex(i8)));
}
void codeDD86(){
	UINT8 i8;
	//ADD   A,(IX+d)
	setT19();
	z80ADD(readMemory(regIX+getCodeIndex(i8)));
}
void codeDD8E(){
	UINT8 i8;
	//ADC   A,(IX+d)
	setT19();
	z80ADC(readMemory(regIX+getCodeIndex(i8)));
}
void codeDD96(){
	UINT8 i8;
	//SUB   A,(IX+d)
	setT19();
	z80SUB(readMemory(regIX+getCodeIndex(i8)));
}
void codeDD9E(){
	UINT8 i8;
	//SBC   A,(IX+d)
	setT19();
	z80SBC(readMemory(regIX+getCodeIndex(i8)));
}
void codeDDA6(){
	UINT8 i8;
	//AND   (IX+d)
	setT19();
	z80AND(readMemory(regIX+getCodeIndex(i8)));
}
void codeDDAE(){
	UINT8 i8;
	//XOR   (IX+d)
	setT19();
	z80XOR(readMemory(regIX+getCodeIndex(i8)));
}
void codeDDB6(){
	UINT8 i8;
	//OR    (IX+d)
	setT19();
	z80OR(readMemory(regIX+getCodeIndex(i8)));
}
void codeDDBE(){
	UINT8 i8;
	//CP    (IX+d)
	setT19();
	z80CP(readMemory(regIX+getCodeIndex(i8)));
}
void codeDDCB06(UINT16 i16){
	//RLC   (IX+d)
	setT23();
	writeMemory(i16,z80RLC(readMemory(i16)));
}
void codeDDCB0E(UINT16 i16){
	//RRC   (IX+d)
	setT23();
	writeMemory(i16,z80RRC(readMemory(i16)));
}
void codeDDCB16(UINT16 i16){
	//RL    (IX+d)
	setT23();
	writeMemory(i16,z80RL(readMemory(i16)));
}
void codeDDCB1E(UINT16 i16){
	//RR    (IX+d)
	setT23();
	writeMemory(i16,z80RR(readMemory(i16)));
}
void codeDDCB26(UINT16 i16){
	//SLA   (IX+d)
	setT23();
	writeMemory(i16,z80SLA(readMemory(i16)));
}
void codeDDCB2E(UINT16 i16){
	//SRA   (IX+d)
	setT23();
	writeMemory(i16,z80SRA(readMemory(i16)));
}
void codeDDCB3E(UINT16 i16){
	//SRL   (IX+d)
	setT23();
	writeMemory(i16,z80SRL(readMemory(i16)));
}
void codeDDCB46(UINT16 i16){
	//BIT   0,(IX+d)
	setT20();
	if (0x01&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeDDCB4E(UINT16 i16){
	//BIT   1,(IX+d)
	setT20();
	if (0x02&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeDDCB56(UINT16 i16){
	//BIT   2,(IX+d)
	setT20();
	if (0x04&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeDDCB5E(UINT16 i16){
	//BIT   3,(IX+d)
	setT20();
	if (0x08&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeDDCB66(UINT16 i16){
	//BIT   4,(IX+d)
	setT20();
	if (0x10&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeDDCB6E(UINT16 i16){
	//BIT   5,(IX+d)
	setT20();
	if (0x20&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeDDCB76(UINT16 i16){
	//BIT   6,(IX+d)
	setT20();
	if (0x40&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeDDCB7E(UINT16 i16){
	//BIT   7,(IX+d)
	setT20();
	if (0x80&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeDDCB86(UINT16 i16){
	//RES   0,(IX+d)
	setT23();
	writeMemory(i16,0xFE&readMemory(i16));
}
void codeDDCB8E(UINT16 i16){
	//RES   1,(IX+d)
	setT23();
	writeMemory(i16,0xFD&readMemory(i16));
}
void codeDDCB96(UINT16 i16){
	//RES   2,(IX+d)
	setT23();
	writeMemory(i16,0xFB&readMemory(i16));
}
void codeDDCB9E(UINT16 i16){
	//RES   3,(IX+d)
	setT23();
	writeMemory(i16,0xF7&readMemory(i16));
}
void codeDDCBA6(UINT16 i16){
	//RES   4,(IX+d)
	setT23();
	writeMemory(i16,0xEF&readMemory(i16));
}
void codeDDCBAE(UINT16 i16){
	//RES   5,(IX+d)
	setT23();
	writeMemory(i16,0xDF&readMemory(i16));
}
void codeDDCBB6(UINT16 i16){
	//RES   6,(IX+d)
	setT23();
	writeMemory(i16,0xBF&readMemory(i16));
}
void codeDDCBBE(UINT16 i16){
	//RES   7,(IX+d)
	setT23();
	writeMemory(i16,0x7F&readMemory(i16));
}
void codeDDCBC6(UINT16 i16){
	//SET   0,(IX+d)
	setT23();
	writeMemory(i16,0x01|readMemory(i16));
}
void codeDDCBCE(UINT16 i16){
	//SET   1,(IX+d)
	setT23();
	writeMemory(i16,0x02|readMemory(i16));
}
void codeDDCBD6(UINT16 i16){
	//SET   2,(IX+d)
	setT23();
	writeMemory(i16,0x04|readMemory(i16));
}
void codeDDCBDE(UINT16 i16){
	//SET   3,(IX+d)
	setT23();
	writeMemory(i16,0x08|readMemory(i16));
}
void codeDDCBE6(UINT16 i16){
	//SET   4,(IX+d)
	setT23();
	writeMemory(i16,0x10|readMemory(i16));
}
void codeDDCBEE(UINT16 i16){
	//SET   5,(IX+d)
	setT23();
	writeMemory(i16,0x20|readMemory(i16));
}
void codeDDCBF6(UINT16 i16){
	//SET   6,(IX+d)
	setT23();
	writeMemory(i16,0x40|readMemory(i16));
}
void codeDDCBFE(UINT16 i16){
	//SET   7,(IX+d)
	setT23();
	writeMemory(i16,0x80|readMemory(i16));
}
void codeDDE1(){
	//POP   IX
	setT14();
	loadIXl(readMemory(regSP));
	loadIXh(readMemory(regSP+1));
	loadSP(regSP+2);
}
void codeDDE3(){
	UINT8 i8;
	UINT8 i82;
	//DD E3        EX    (SP),IX
	setT23();
	i8=readMemory(regSP);
	i82=readMemory(regSP+1);
	writeMemory(regSP,regIXl);
	writeMemory(regSP+1,regIXh);
	loadIXl(i8);
	loadIXh(i82);
}
void codeDDE5(){
	//PUSH   IX
	setT15();
	loadSP(regSP-2);
	writeMemory(regSP,regIXl);
	writeMemory(regSP+1,regIXh);
}
void codeDDE9(){
	//JP    (IX)
	setT8();
	loadPC(regIX);
}
void codeDDF9(){
	//LD    SP,IX
	setT10();
	loadSP(regIX);
}
void codeED40(){
	//IN    B,(C)
	UINT8 i8,j8;
	setT12();
	i8=readIO(regC, regB);
	loadB(i8);
	if (i8&0x80) setSflag();
	else clearSflag();
	if (i8) clearZflag();
	else setZflag();
	clearHflag();
	setZ80Parity(i8,j8);
	clearNflag();
}
void codeED41(){
	//OUT   (C),B
	setT12();
	writeIO(regC, regB, regB);
}
void codeED42(){
	//SBC   HL,BC
	setT15();
	z80SBC16(regBC);
}
void codeED43(){
	UINT16 i16;
	//LD    (nn),BC
	setT20();
	i16=getCode();
	i16|=getCode()<<8;
	writeMemory(i16,regC);
	writeMemory(i16+1,regB);
}
void codeED44(){
	UINT8 i8;
	//NEG
	setT8();
	if (0x80==regA) setPflag();
	else clearPflag();
	i8=0-regA;
	if ((i8^regA)&0x10) setHflag();
	else clearHflag();
	loadA(i8);
	if (0x80&i8) setSflag();
	else clearSflag();
	if (i8) {
		clearZflag();
		setCflag();
	} else {
		setZflag();
		clearCflag();
	}
}
void codeED45(){
	UINT8 i8;
	UINT16 i16;
	//RETN
	setT14();
	i8=readMemory(regSP);
	i16=readMemory(regSP+1)<<8;
	i16|=i8;
	loadPC(i16);
	loadSP(regSP+2);
	loadIFF1(flagIFF2);
}
void codeED46(){
	//IM    0
	setT8();
	loadIM(0);
}
void codeED47(){
	//LD    I,A
	setT9();
	loadI(regA);
}
void codeED48(){
	//IN    C,(C)
	UINT8 i8,j8;
	setT12();
	i8=readIO(regC, regB);
	loadC(i8);
	if (i8&0x80) setSflag();
	else clearSflag();
	if (i8) clearZflag();
	else setZflag();
	clearHflag();
	setZ80Parity(i8,j8);
	clearNflag();
}
void codeED49(){
	//OUT   (C),C
	setT12();
	writeIO(regC, regB, regC);
}
void codeED4A(){
	//ADC   HL,BC
	setT15();
	z80ADC16(regBC);
}
void codeED4B(){
	UINT16 i16;
	//LD    BC,(nn)
	setT20();
	i16=getCode();
	i16|=getCode()<<8;
	loadC(readMemory(i16));
	loadB(readMemory(i16+1));
}
void codeED4D(){
	UINT8 i8;
	UINT16 i16;
	//RETI
	setT14();
	i8=readMemory(regSP);
	i16=readMemory(regSP+1)<<8;
	i16|=i8;
	loadPC(i16);
	loadSP(regSP+2);
}
void codeED4F(){
	//LD    R,A
	setT9();
	loadR(regA);
}
void codeED50(){
	//IN    D,(C)
	UINT8 i8,j8;
	setT12();
	i8=readIO(regC, regB);
	loadD(i8);
	if (i8&0x80) setSflag();
	else clearSflag();
	if (i8) clearZflag();
	else setZflag();
	clearHflag();
	setZ80Parity(i8,j8);
	clearNflag();
}
void codeED51(){
	//OUT   (C),D
	setT12();
	writeIO(regC, regB, regD);
}
void codeED52(){
	//SBC   HL,DE
	setT15();
	z80SBC16(regDE);
}
void codeED53(){
	UINT16 i16;
	//LD    (nn),DE
	setT20();
	i16=getCode();
	i16|=getCode()<<8;
	writeMemory(i16,regE);
	writeMemory(i16+1,regD);
}
void codeED56(){
	//IM    1
	setT8();
	loadIM(1);
}
void codeED57(){
	//ED 57   LD    A,I
	setT9();
	loadA(regI);
	if (flagIFF2) setPflag();
	else clearPflag();
}
void codeED58(){
	//IN    E,(C)
	UINT8 i8,j8;
	setT12();
	i8=readIO(regC, regB);
	loadE(i8);
	if (i8&0x80) setSflag();
	else clearSflag();
	if (i8) clearZflag();
	else setZflag();
	clearHflag();
	setZ80Parity(i8,j8);
	clearNflag();
}
void codeED59(){
	//OUT   (C),E
	setT12();
	writeIO(regC, regB, regE);
}
void codeED5A(){
	//ADC   HL,DE
	setT15();
	z80ADC16(regDE);
}
void codeED5B(){
	UINT16 i16;
	//LD    DE,(nn)
	setT20();
	i16=getCode();
	i16|=getCode()<<8;
	loadE(readMemory(i16));
	loadD(readMemory(i16+1));
}
void codeED5E(){
	//IM    2
	setT8();
	loadIM(2);
}
void codeED5F(){
	//ED 5F   LD    A,R
	setT9();
	loadA(regR);
	if (flagIFF2) setPflag();
	else clearPflag();
}
void codeED60(){
	//IN    H,(C)
	UINT8 i8,j8;
	setT12();
	i8=readIO(regC, regB);
	loadH(i8);
	if (i8&0x80) setSflag();
	else clearSflag();
	if (i8) clearZflag();
	else setZflag();
	clearHflag();
	setZ80Parity(i8,j8);
	clearNflag();
}
void codeED61(){
	//OUT   (C),H
	setT12();
	writeIO(regC, regB, regH);
}
void codeED62(){
	//SBC   HL,HL
	setT15();
	z80SBC16(regHL);
}
void codeED63(){
	UINT16 i16;
	//LD (nn),HL
	i16=getCode();
	i16|=getCode()<<8;
	writeMemory(i16,regL);
	writeMemory(i16+1,regH);
}
void codeED67(){
	UINT8 i8;
	//RRD
	i8=readMemory(regHL);
	writeMemory(regHL,(i8>>4)|(regA<<4));
	loadA(regA&0xf0 | i8&0x0f);
	if (regA&0x80) setSflag();
	else clearSflag();
	if (regA) clearZflag();
	else setZflag();
	clearHflag();
	clearNflag();
	setZ80Parity(regA,i8);
}
void codeED68(){
	//IN    L,(C)
	UINT8 i8,j8;
	setT12();
	i8=readIO(regC, regB);
	loadL(i8);
	if (i8&0x80) setSflag();
	else clearSflag();
	if (i8) clearZflag();
	else setZflag();
	clearHflag();
	setZ80Parity(i8,j8);
	clearNflag();
}
void codeED69(){
	//OUT   (C),L
	setT12();
	writeIO(regC, regB, regL);
}
void codeED6A(){
	//ADC   HL,HL
	setT15();
	z80ADC16(regHL);
}
void codeED6B(){
	UINT16 i16;
	//LD HL,(nn)
	i16=getCode();
	i16|=getCode()<<8;
	loadL(readMemory(i16));
	loadH(readMemory(i16+1));
}
void codeED6F(){
	UINT8 i8;
	//RLD
	i8=readMemory(regHL);
	writeMemory(regHL,(i8<<4)|(regA&0x0f));
	loadA(regA&0xf0 | (i8>>4));
	if (regA&0x80) setSflag();
	else clearSflag();
	if (regA) clearZflag();
	else setZflag();
	clearHflag();
	clearNflag();
	setZ80Parity(regA,i8);
}
void codeED72(){
	//SBC   HL,SP
	setT15();
	z80SBC16(regSP);
}
void codeED73(){
	UINT16 i16;
	//LD    (nn),SP
	setT20();
	i16=getCode();
	i16|=getCode()<<8;
	writeMemory(i16,regSPl);
	writeMemory(i16+1,regSPh);
}
void codeED78(){
	//IN    A,(C)
	UINT8 i8,j8;
	setT12();
	i8=readIO(regC, regB);
	loadA(i8);
	if (i8&0x80) setSflag();
	else clearSflag();
	if (i8) clearZflag();
	else setZflag();
	clearHflag();
	setZ80Parity(i8,j8);
	clearNflag();
}
void codeED79(){
	//OUT   (c),A
	setT12();
	writeIO(regC, regB, regA);
}
void codeED7A(){
	//ADC   HL,SP
	setT15();
	z80ADC16(regSP);
}
void codeED7B(){
	UINT16 i16;
	//LD    SP,(nn)
	setT20();
	i16=getCode();
	i16|=getCode()<<8;
	loadSPl(readMemory(i16));
	loadSPh(readMemory(i16+1));
}
void codeEDA0(){
	//LDI
	//(DE)  (HL), DE  DE + 1, HL  HL + 1, BC  BC -1
	setT16();
	writeMemory(regDE,readMemory(regHL));
	loadDE(regDE+1);
	loadHL(regHL+1);
	loadBC(regBC-1);
	clearHflag();
	if (regBC) setPflag();
	else clearPflag();
	clearNflag();
}
void codeEDA1(){
	UINT8 i8;
	UINT8 i82;
	//CPI
	//A- (HL), HL  HL +1, BC  BC -1
	setT16();
	i8=readMemory(regHL);
	i82=(0x0F&regA)-(0x0F&i8);
	i8=regA-i8;
	loadHL(regHL+1);
	loadBC(regBC-1);
	if (0x80&i8) setSflag();
	else clearSflag();
	if (i8) clearZflag();
	else setZflag();
	if (0x80&i82) setHflag();
	else clearHflag();
	if (regBC) setPflag();
	else clearPflag();
	setNflag();
}
void codeEDA2(){
	//INI
	//(HL)  (C), B  B -1, HL  HL + 1
	setT16();
	writeMemory(regHL,readIO(regC,regB));
	loadHL(regHL+1);
	loadB(regB-1);
	if (regB) clearZflag();
	else setZflag();
	setNflag();
}
void codeEDA3(){
	//OUTI
	//(C)  (HL), B  B -1, HL  HL + 1
	setT16();
	writeIO(regC,regB,readMemory(regHL));
	loadHL(regHL+1);
	loadB(regB-1);
	if (regB) clearZflag();
	else setZflag();
	setNflag();	
}
void codeEDA8(){
	//LDD
	//(DE)  (HL), DE  DE -1, HL  HL-1, BC  BC-1
	setT16();
	writeMemory(regDE,readMemory(regHL));
	loadDE(regDE-1);
	loadHL(regHL-1);
	loadBC(regBC-1);
	clearHflag();
	if (regBC) setPflag();
	else clearPflag();
	clearNflag();
}
void codeEDA9(){
	UINT8 i8;
	UINT8 i82;
	//CPD
	//A -(HL), HL  HL -1, BC  BC -1
	setT16();
	i8=readMemory(regHL);
	i82=(0x0F&regA)-(0x0F&i8);
	i8=regA-i8;
	loadHL(regHL-1);
	loadBC(regBC-1);
	if (0x80&i8) setSflag();
	else clearSflag();
	if (i8) clearZflag();
	else setZflag();
	if (0x80&i82) setHflag();
	else clearHflag();
	if (regBC) setPflag();
	else clearPflag();
	setNflag();
}
void codeEDAA(){
	//IND
	//(HL)  (C), B  B -1, HL  HL -1
	setT16();
	writeMemory(regHL,readIO(regC,regB));
	loadHL(regHL-1);
	loadB(regB-1);
	if (regB) clearZflag();
	else setZflag();
	setNflag();
}
void codeEDAB(){
	//OUTD
	//(C)  (HL), B  B -1, HL  HL -1
	setT16();
	writeIO(regC,regB,readMemory(regHL));
	loadHL(regHL-1);
	loadB(regB-1);
	if (regB) clearZflag();
	else setZflag();
	setNflag();	
}
void codeEDB0(){
	//LDIR
	//(DE)  (HL), DE  DE + 1, HL  HL + 1, BC F? BC -1
	if (regBC==1) {
		setT16();
	} else {
		setT21();
		loadPC(regPC-2);
	}
	writeMemory(regDE,readMemory(regHL));
	loadDE(regDE+1);
	loadHL(regHL+1);
	loadBC(regBC-1);
	clearHflag();
	clearPflag();
	clearNflag();
}
void codeEDB1(){
	UINT8 i8;
	UINT8 i82;
	//CPIR
	//A-(HL), HL  HL+1, BC  BC-1
	if (regBC==1 || regA==readMemory(regHL)) {
		setT16();
	} else {
		setT21();
		loadPC(regPC-2);
	}
	i8=readMemory(regHL);
	i82=(0x0F&regA)-(0x0F&i8);
	i8=regA-i8;
	loadHL(regHL+1);
	loadBC(regBC-1);
	if (0x80&i8) setSflag();
	else clearSflag();
	if (i8) clearZflag();
	else setZflag();
	if (0x80&i82) setHflag();
	else clearHflag();
	if (regBC) setPflag();
	else clearPflag();
	setNflag();
}
void codeEDB2(){
	//INIR
	//(HL)  (C), B  B -1, HL  HL +1
	if (regB==1) {
		setT16();
	} else {
		setT21();
		loadPC(regPC-2);
	}
	writeMemory(regHL,readIO(regC,regB));
	loadHL(regHL+1);
	loadB(regB-1);
	setZflag();
	setNflag();
}
void codeEDB3(){
	//OTIR
	//(C)  (HL), B  B -1, HL  HL + 1
	if (regB==1) {
		setT16();
	} else {
		setT21();
		loadPC(regPC-2);
	}
	writeIO(regC,regB,readMemory(regHL));
	loadHL(regHL+1);
	loadB(regB-1);
	setZflag();
	setNflag();	
}
void codeEDB8(){
	//LDDR
	if (regBC==1) {
		setT16();
	} else {
		setT21();
		loadPC(regPC-2);
	}
	writeMemory(regDE,readMemory(regHL));
	loadDE(regDE-1);
	loadHL(regHL-1);
	loadBC(regBC-1);
	clearHflag();
	clearPflag();
	clearNflag();
}
void codeEDB9(){
	UINT8 i8;
	UINT8 i82;
	//CPDR
	if (regBC==1 || regA==readMemory(regHL)) {
		setT16();
	} else {
		setT21();
		loadPC(regPC-2);
	}
	i8=readMemory(regHL);
	i82=(0x0F&regA)-(0x0F&i8);
	i8=regA-i8;
	loadHL(regHL-1);
	loadBC(regBC-1);
	if (0x80&i8) setSflag();
	else clearSflag();
	if (i8) clearZflag();
	else setZflag();
	if (0x80&i82) setHflag();
	else clearHflag();
	if (regBC) setPflag();
	else clearPflag();
	setNflag();
}
void codeEDBA(){
	//INDR
	if (regB==1) {
		setT16();
	} else {
		setT21();
		loadPC(regPC-2);
	}
	writeMemory(regHL,readIO(regC,regB));
	loadHL(regHL-1);
	loadB(regB-1);
	setZflag();
	setNflag();
}
void codeEDBB(){
	//OTDR
	if (regB==1) {
		setT16();
	} else {
		setT21();
		loadPC(regPC-2);
	}
	writeIO(regC,regB,readMemory(regHL));
	loadHL(regHL-1);
	loadB(regB-1);
	setZflag();
	setNflag();	
}
void codeFD09(){
	UINT16 i16;
	UINT32 i32;
	//ADD   IY,BC
	setT15();
	i32=regIY+regBC;
	i16=(regIY&0x0FFF)+(regBC&0x0FFF);
	loadIY(i32);
	if (0xFFFF<i32) setCflag();
	else clearCflag();
	if (0x0FFF<i16) setHflag();
	else clearHflag();
	clearNflag();
}
void codeFD19(){
	UINT16 i16;
	UINT32 i32;
	//ADD   IY,DE
	setT15();
	i32=regIY+regDE;
	i16=(regIY&0x0FFF)+(regDE&0x0FFF);
	loadIY(i32);
	if (0xFFFF<i32) setCflag();
	else clearCflag();
	if (0x0FFF<i16) setHflag();
	else clearHflag();
	clearNflag();
}
void codeFD21(){
	UINT16 i16;
	//LD    IY,nn
	setT14();
	i16=getCode();
	i16|=getCode()<<8;
	loadIY(i16);
}
void codeFD22(){
	UINT16 i16;
	//LD    (nn),IY
	setT20();
	i16=getCode();
	i16|=getCode()<<8;
	writeMemory(i16,regIYl);
	writeMemory(i16+1,regIYh);
}
void codeFD23(){
	//INC   IY
	setT10();
	loadIY(regIY+1);
}
void codeFD29(){
	UINT16 i16;
	UINT32 i32;
	//ADD   IY,IY
	setT15();
	i32=regIY<<1;
	i16=(regIY&0x0FFF)<<1;
	loadIY(i32);
	if (0xFFFF<i32) setCflag();
	else clearCflag();
	if (0x0FFF<i16) setHflag();
	else clearHflag();
	clearNflag();
}
void codeFD2A(){
	UINT16 i16;
	//LD    IY,(nn)
	setT20();
	i16=getCode();
	i16|=getCode()<<8;
	loadIYl(readMemory(i16));
	loadIYh(readMemory(i16+1));
}
void codeFD2B(){
	//DEC   IY
	setT10();
	loadIY(regIY-1);
}
void codeFD34(){
	UINT8 i8;
	UINT16 i16;
	//INC   (IY+d)
	setT23();
	i16=regIY+getCodeIndex(i8);
	i8=readMemory(i16)+1;
	writeMemory(i16,i8);
	flag8inc(i8);
}
void codeFD35(){
	UINT8 i8;
	UINT16 i16;
	//DEC   (IY+D)
	setT23();
	i16=regIY+getCodeIndex(i8);
	i8=readMemory(i16)-1;
	writeMemory(i16,i8);
	flag8dec(i8);
}
void codeFD36(){
	UINT8 i8;
	UINT16 i16;
	//LD    (IY+d),n
	setT19();
	i16=regIY+getCodeIndex(i8);
	writeMemory(i16,getCode());
}
void codeFD39(){
	UINT16 i16;
	UINT32 i32;
	//ADD   IY,SP
	setT15();
	i32=regIY+regSP;
	i16=(regIY&0x0FFF)+(regSP&0x0FFF);
	loadIY(i32);
	if (0xFFFF<i32) setCflag();
	else clearCflag();
	if (0x0FFF<i16) setHflag();
	else clearHflag();
	clearNflag();
}
void codeFD46(){
	UINT8 i8;
	//LD    B,(IY+d)
	setT19();
	loadB(readMemory(regIY+getCodeIndex(i8)));
}
void codeFD4E(){
	UINT8 i8;
	//LD    C,(IY+d)
	setT19();
	loadC(readMemory(regIY+getCodeIndex(i8)));
}
void codeFD56(){
	UINT8 i8;
	//LD    D,(IY+d)
	setT19();
	loadD(readMemory(regIY+getCodeIndex(i8)));
}
void codeFD5E(){
	UINT8 i8;
	//LD    E,(IY+d)
	setT19();
	loadE(readMemory(regIY+getCodeIndex(i8)));
}
void codeFD66(){
	UINT8 i8;
	//LD    H,(IY+d)
	setT19();
	loadH(readMemory(regIY+getCodeIndex(i8)));
}
void codeFD6E(){
	UINT8 i8;
	//LD    L,(IY+d)
	setT19();
	loadL(readMemory(regIY+getCodeIndex(i8)));
}
void codeFD70(){
	UINT8 i8;
	//LD    (IY+d),B
	setT19();
	writeMemory(regIY+getCodeIndex(i8),regB);
}
void codeFD71(){
	UINT8 i8;
	//LD    (IY+d),C
	setT19();
	writeMemory(regIY+getCodeIndex(i8),regC);
}
void codeFD72(){
	UINT8 i8;
	//LD    (IY+d),D
	setT19();
	writeMemory(regIY+getCodeIndex(i8),regD);
}
void codeFD73(){
	UINT8 i8;
	//LD    (IY+d),E
	setT19();
	writeMemory(regIY+getCodeIndex(i8),regE);
}
void codeFD74(){
	UINT8 i8;
	//LD    (IY+d),H
	setT19();
	writeMemory(regIY+getCodeIndex(i8),regH);
}
void codeFD75(){
	UINT8 i8;
	//LD    (IY+d),L
	setT19();
	writeMemory(regIY+getCodeIndex(i8),regL);
}
void codeFD77(){
	UINT8 i8;
	//LD    (IY+d),A
	setT19();
	writeMemory(regIY+getCodeIndex(i8),regA);
}
void codeFD7E(){
	UINT8 i8;
	//LD    A,(IY+d)
	setT19();
	loadA(readMemory(regIY+getCodeIndex(i8)));
}
void codeFD86(){
	UINT8 i8;
	//ADD   A,(IY+d)
	setT19();
	z80ADD(readMemory(regIY+getCodeIndex(i8)));
}
void codeFD8E(){
	UINT8 i8;
	//ADC   A,(IY+d)
	setT19();
	z80ADC(readMemory(regIY+getCodeIndex(i8)));
}
void codeFD96(){
	UINT8 i8;
	//SUB   A,(IY+d)
	setT19();
	z80SUB(readMemory(regIY+getCodeIndex(i8)));
}
void codeFD9E(){
	UINT8 i8;
	//SBC   A,(IY+d)
	setT19();
	z80SBC(readMemory(regIY+getCodeIndex(i8)));
}
void codeFDA6(){
	UINT8 i8;
	//AND   (IY+d)
	setT19();
	z80AND(readMemory(regIY+getCodeIndex(i8)));
}
void codeFDAE(){
	UINT8 i8;
	//XOR   (IY+d)
	setT19();
	z80XOR(readMemory(regIY+getCodeIndex(i8)));
}
void codeFDB6(){
	UINT8 i8;
	//OR    (IY+d)
	setT19();
	z80OR(readMemory(regIY+getCodeIndex(i8)));
}
void codeFDBE(){
	UINT8 i8;
	//CP    (IY+d)
	setT19();
	z80CP(readMemory(regIY+getCodeIndex(i8)));
}
void codeFDCB06(UINT16 i16){
	//RLC   (IY+d)
	setT23();
	writeMemory(i16,z80RLC(readMemory(i16)));
}
void codeFDCB0E(UINT16 i16){
	//RRC   (IY+d)
	setT23();
	writeMemory(i16,z80RRC(readMemory(i16)));
}
void codeFDCB16(UINT16 i16){
	//RL    (IY+d)
	setT23();
	writeMemory(i16,z80RL(readMemory(i16)));
}
void codeFDCB1E(UINT16 i16){
	//RR    (IY+d)
	setT23();
	writeMemory(i16,z80RR(readMemory(i16)));
}
void codeFDCB26(UINT16 i16){
	//SLA   (IY+d)
	setT23();
	writeMemory(i16,z80SLA(readMemory(i16)));
}
void codeFDCB2E(UINT16 i16){
	//SRA   (IY+d)
	setT23();
	writeMemory(i16,z80SRA(readMemory(i16)));
}
void codeFDCB3E(UINT16 i16){
	//SRL   (IY+d)
	setT23();
	writeMemory(i16,z80SRL(readMemory(i16)));
}
void codeFDCB46(UINT16 i16){
	//BIT   0,(IY+d)
	setT20();
	if (0x01&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeFDCB4E(UINT16 i16){
	//BIT   1,(IY+d)
	setT20();
	if (0x02&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeFDCB56(UINT16 i16){
	//BIT   2,(IY+d)
	setT20();
	if (0x04&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeFDCB5E(UINT16 i16){
	//BIT   3,(IY+d)
	setT20();
	if (0x08&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeFDCB66(UINT16 i16){
	//BIT   4,(IY+d)
	setT20();
	if (0x10&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeFDCB6E(UINT16 i16){
	//BIT   5,(IY+d)
	setT20();
	if (0x20&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeFDCB76(UINT16 i16){
	//BIT   6,(IY+d)
	setT20();
	if (0x40&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeFDCB7E(UINT16 i16){
	//BIT   7,(IY+d)
	setT20();
	if (0x80&readMemory(i16)) clearZflag();
	else setZflag();
	setHflag();
	clearNflag();
}
void codeFDCB86(UINT16 i16){
	//RES   0,(IY+d)
	setT23();
	writeMemory(i16,0xFE&readMemory(i16));
}
void codeFDCB8E(UINT16 i16){
	//RES   1,(IY+d)
	setT23();
	writeMemory(i16,0xFD&readMemory(i16));
}
void codeFDCB96(UINT16 i16){
	//RES   2,(IY+d)
	setT23();
	writeMemory(i16,0xFB&readMemory(i16));
}
void codeFDCB9E(UINT16 i16){
	//RES   3,(IY+d)
	setT23();
	writeMemory(i16,0xF7&readMemory(i16));
}
void codeFDCBA6(UINT16 i16){
	//RES   4,(IY+d)
	setT23();
	writeMemory(i16,0xEF&readMemory(i16));
}
void codeFDCBAE(UINT16 i16){
	//RES   5,(IY+d)
	setT23();
	writeMemory(i16,0xDF&readMemory(i16));
}
void codeFDCBB6(UINT16 i16){
	//RES   6,(IY+d)
	setT23();
	writeMemory(i16,0xBF&readMemory(i16));
}
void codeFDCBBE(UINT16 i16){
	//RES   7,(IY+d)
	setT23();
	writeMemory(i16,0x7F&readMemory(i16));
}
void codeFDCBC6(UINT16 i16){
	//SET   0,(IY+d)
	setT23();
	writeMemory(i16,0x01|readMemory(i16));
}
void codeFDCBCE(UINT16 i16){
	//SET   1,(IY+d)
	setT23();
	writeMemory(i16,0x02|readMemory(i16));
}
void codeFDCBD6(UINT16 i16){
	//SET   2,(IY+d)
	setT23();
	writeMemory(i16,0x04|readMemory(i16));
}
void codeFDCBDE(UINT16 i16){
	//SET   3,(IY+d)
	setT23();
	writeMemory(i16,0x08|readMemory(i16));
}
void codeFDCBE6(UINT16 i16){
	//SET   4,(IY+d)
	setT23();
	writeMemory(i16,0x10|readMemory(i16));
}
void codeFDCBEE(UINT16 i16){
	//SET   5,(IY+d)
	setT23();
	writeMemory(i16,0x20|readMemory(i16));
}
void codeFDCBF6(UINT16 i16){
	//SET   6,(IY+d)
	setT23();
	writeMemory(i16,0x40|readMemory(i16));
}
void codeFDCBFE(UINT16 i16){
	//SET   7,(IY+d)
	setT23();
	writeMemory(i16,0x80|readMemory(i16));
}
void codeFDE1(){
	//POP   IY
	setT14();
	loadIYl(readMemory(regSP));
	loadIYh(readMemory(regSP+1));
	loadSP(regSP+2);
}
void codeFDE3(){
	UINT8 i8;
	UINT8 i82;
	//DD E3        EX    (SP),IY
	setT23();
	i8=readMemory(regSP);
	i82=readMemory(regSP+1);
	writeMemory(regSP,regIYl);
	writeMemory(regSP+1,regIYh);
	loadIYl(i8);
	loadIYh(i82);
}
void codeFDE5(){
	//PUSH   IY
	setT15();
	loadSP(regSP-2);
	writeMemory(regSP,regIYl);
	writeMemory(regSP+1,regIYh);
}
void codeFDE9(){
	//JP    (IY)
	setT8();
	loadPC(regIY);
}
void codeFDF9(){
	//LD    SP,IY
	setT10();
	loadSP(regIY);
}
void codeCB();
void codeDD();
void codeED();
void codeFD();
void codeDDCB();
void codeFDCB();
#endif //Z80
#include "z80jump.h"
